package cn.com.poc.audio_generation.aggregation.impl;

import cn.com.poc.audio_generation.aggregation.AudioGenerationService;
import cn.com.poc.audio_generation.constant.AudioGenerationTaskStatusConstant;
import cn.com.poc.audio_generation.entity.BatchGenerateEntity;
import cn.com.poc.audio_generation.entity.BizAudioGenerationTaskEntity;
import cn.com.poc.audio_generation.entity.GenerationStatusEntity;
import cn.com.poc.audio_generation.service.BizAudioGenerationTaskService;
import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.message.entity.AudioGenerationMessage;
import cn.com.poc.message.service.AudioGenerationProducerService;
import cn.com.yict.framemax.core.exception.BusinessException;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Component
public class AudioGenerationServiceImpl implements AudioGenerationService {


    @Resource
    private BizAudioGenerationTaskService bizAudioGenerationTaskService;

    @Resource
    private AudioGenerationProducerService audioGenerationProducerService;

    @Override
    public Long createTextToAudioTask(String text, String voiceId, Double speed) {
        Assert.notBlank(text, "text is blank");
        Assert.notBlank(voiceId, "voiceId is blank");
        BizAudioGenerationTaskEntity bizAudioGenerationTaskEntity = new BizAudioGenerationTaskEntity();
        bizAudioGenerationTaskEntity.setTextContent(text);
        bizAudioGenerationTaskEntity.setVoiceId(voiceId);
        bizAudioGenerationTaskEntity.setTaskStatus(AudioGenerationTaskStatusConstant.Running);
        bizAudioGenerationTaskEntity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizAudioGenerationTaskEntity taskEntity = bizAudioGenerationTaskService.save(bizAudioGenerationTaskEntity);

        // 创建任务后，需要调用语音合成服务，生成音频文件
        AudioGenerationMessage audioGenerationMessage = new AudioGenerationMessage();
        audioGenerationMessage.setTaskId(taskEntity.getTaskId());
        audioGenerationMessage.setTextContent(text);
        audioGenerationMessage.setVoiceId(voiceId);
        audioGenerationMessage.setSpeed(speed);
        audioGenerationProducerService.generateAudio(audioGenerationMessage);

        return taskEntity.getTaskId();
    }

    @Override
    public List<Long> batchCreateTextToAudioTask(List<BatchGenerateEntity> entities) {
        Assert.notEmpty(entities, "entities is empty");
        List<Long> taskIds = new ArrayList<>();
        entities.forEach(entity -> {
            taskIds.add(createTextToAudioTask(entity.getTextContent(), entity.getVoiceId(), entity.getSpeed()));
        });
        return taskIds;
    }

    @Override
    public String getTaskStatus(Long taskId) {
        BizAudioGenerationTaskEntity bizAudioGenerationTaskEntity = bizAudioGenerationTaskService.get(taskId);
        if (null == bizAudioGenerationTaskEntity) {
            throw new BusinessException("taskId is not exist");
        }
        return bizAudioGenerationTaskEntity.getTaskStatus();
    }

    @Override
    public List<GenerationStatusEntity> batchGetAudioGenerationTask(List<Long> taskIds) {
        Assert.notEmpty(taskIds, "taskIds is empty");
        List<GenerationStatusEntity> generationStatusEntities = new ArrayList<>();
        for (Long taskId : taskIds) {
            BizAudioGenerationTaskEntity bizAudioGenerationTaskEntity = bizAudioGenerationTaskService.get(taskId);
            if (null == bizAudioGenerationTaskEntity) {
                throw new BusinessException("taskId is not exist");
            }
            GenerationStatusEntity generationStatusEntity = new GenerationStatusEntity();
            generationStatusEntity.setTaskStatus(bizAudioGenerationTaskEntity.getTaskStatus());
            generationStatusEntity.setTaskId(taskId);
            generationStatusEntity.setAudioUrl(bizAudioGenerationTaskEntity.getAudioUrl());
            generationStatusEntities.add(generationStatusEntity);
        }
        return generationStatusEntities;
    }
}
