package cn.com.poc.audio_generation.rest.impl;

import cn.com.poc.audio_generation.aggregation.AudioGenerationService;
import cn.com.poc.audio_generation.dto.BatchGenerateDto;
import cn.com.poc.audio_generation.dto.GenerationStatusDto;
import cn.com.poc.audio_generation.entity.BatchGenerateEntity;
import cn.com.poc.audio_generation.entity.GenerationStatusEntity;
import cn.com.poc.audio_generation.rest.GeneratedAudioRest;
import cn.com.poc.common.utils.Assert;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class GeneratedAudioRestImpl implements GeneratedAudioRest {

    @Resource
    private AudioGenerationService audioGenerationService;

    @Override
    public List<Long> batchGenerate(List<BatchGenerateDto> generateDtoList) {
        Assert.notEmpty(generateDtoList, "参数不能为空");
        List<BatchGenerateEntity> entities = generateDtoList.stream().map(dto -> {
            BatchGenerateEntity entity = new BatchGenerateEntity();
            entity.setTextContent(dto.getTextContent());
            entity.setVoiceId(dto.getVoiceId());
            return entity;
        }).collect(Collectors.toList());

        return audioGenerationService.batchCreateTextToAudioTask(entities);
    }

    @Override
    public List<GenerationStatusDto> generatedStatus(List<Long> taskIdList) {
        Assert.notEmpty(taskIdList, "参数不能为空");
        List<GenerationStatusEntity> generationStatusEntities = audioGenerationService.batchGetAudioGenerationTask(taskIdList);
        List<GenerationStatusDto> result = generationStatusEntities.stream().map(entity -> {
            GenerationStatusDto dto = new GenerationStatusDto();
            dto.setTaskId(entity.getTaskId());
            dto.setTaskStatus(entity.getTaskStatus());
            dto.setAudioUrl(entity.getAudioUrl());
            return dto;
        }).collect(Collectors.toList());
        return result;
    }
}
