package cn.com.poc.audio_generation.service.impl;

import cn.com.poc.audio_generation.convert.AudioGenerationTaskConvert;
import cn.com.poc.audio_generation.entity.BizAudioGenerationTaskEntity;
import cn.com.poc.audio_generation.model.BizAudioGenerationTaskModel;
import cn.com.poc.audio_generation.repository.BizAudioGenerationTaskRepository;
import cn.com.poc.audio_generation.service.BizAudioGenerationTaskService;
import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
public class BizAudioGenerationTaskServiceImpl extends BaseServiceImpl
        implements BizAudioGenerationTaskService {

    @Resource
    private BizAudioGenerationTaskRepository repository;

    @Override
    public BizAudioGenerationTaskEntity get(Long id) {
        Assert.notNull(id, "id is null");
        BizAudioGenerationTaskModel model = this.repository.get(id);
        if (model == null || model.getIsDeleted().equals(CommonConstant.IsDeleted.Y)) {
            return null;
        }
        return AudioGenerationTaskConvert.modelToEntity(model);
    }

    @Override
    public List<BizAudioGenerationTaskEntity> findByExample(BizAudioGenerationTaskEntity entity, PagingInfo pagingInfo) {
        List<BizAudioGenerationTaskEntity> result = new ArrayList<>();
        BizAudioGenerationTaskModel model = AudioGenerationTaskConvert.entityToModel(entity);
        List<BizAudioGenerationTaskModel> models = this.repository.findByExample(model, pagingInfo);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(AudioGenerationTaskConvert::modelToEntity).collect(java.util.stream.Collectors.toList());
        }
        return result;
    }

    @Override
    public void deleteById(Long id) {
        Assert.notNull(id, "id is null");
        BizAudioGenerationTaskModel model = this.repository.get(id);
        if (model == null || model.getIsDeleted().equals(CommonConstant.IsDeleted.Y)) {
            return;
        }
        model.setIsDeleted(CommonConstant.IsDeleted.Y);
        this.repository.save(model);
    }

    @Override
    public BizAudioGenerationTaskEntity save(BizAudioGenerationTaskEntity entity) {
        Assert.notNull(entity, "entity is null");
        Assert.notBlank(entity.getTextContent(), "text content is null");
        Assert.notBlank(entity.getTaskStatus(), "task status is null");
        Assert.notBlank(entity.getVoiceId(), "voice id is null");
        entity.setTaskId(null);
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizAudioGenerationTaskModel saveModel = this.repository.save(AudioGenerationTaskConvert.entityToModel(entity));
        return AudioGenerationTaskConvert.modelToEntity(saveModel);
    }

    @Override
    public BizAudioGenerationTaskEntity update(BizAudioGenerationTaskEntity entity) {
        Assert.notNull(entity, "entity is null");
        Assert.notNull(entity.getTaskId(), "task id is null");
        BizAudioGenerationTaskModel model = this.repository.get(entity.getTaskId());
        if (model == null || model.getIsDeleted().equals(CommonConstant.IsDeleted.Y)) {
            return null;
        }
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(entity, model, copyOptions);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        return AudioGenerationTaskConvert.modelToEntity(this.repository.save(model));
    }
}