package cn.com.poc.communication.aggregation.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.communication.aggregation.CharacterCommunicationService;
import cn.com.poc.communication.convert.BizCharacterCommunicationConvert;
import cn.com.poc.communication.entity.BizAiPromptEntity;
import cn.com.poc.communication.entity.BizCharacterCommunicationEntity;
import cn.com.poc.communication.entity.BizCommunicationThemeEntity;
import cn.com.poc.communication.entity.SaveCharacterCommunicationEntity;
import cn.com.poc.communication.service.BizAiPromptService;
import cn.com.poc.communication.service.BizCharacterCommunicationService;
import cn.com.poc.communication.service.BizCommunicationThemeService;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.atomic.AtomicLong;

@Service
public class CharacterCommunicationServiceImpl implements CharacterCommunicationService {
    @Resource
    private BizAiPromptService bizAiPromptService;

    @Resource
    private BizCommunicationThemeService bizCommunicationThemeService;

    @Resource
    private BizCharacterCommunicationService bizCharacterCommunicationService;

    private static AtomicLong counter = new AtomicLong(System.currentTimeMillis());

    @Override
    public void sortCharacter(Integer sourceId, Integer targetId) throws Exception {
        //上移时: 1.target（包括自己）~ 后面的sort都+1 2.sourceEntity的sort值变为target的sort   1,target2,3,source4,5 -> 1,source4,target2,3,5 -> (source4->target2) (target2+1->3) (3+1 -> 4)  1,2,3,4,5
        //下移时：1.source（不包括自己）~ targe（包括自己）t的sort都-1 2.sourceEntity的sort值变为target的sort   1,source2,3,target4,5 -> 1,3,target4,source2,5 -> (source2->target4) (target4-1->3) (3-1 -> 2)  1,2,3,4,5

        //首先获取源和目标的sort排序值
        BizCharacterCommunicationEntity sourceEntity = bizCharacterCommunicationService.get(sourceId);
        BizCharacterCommunicationEntity targetEntity = bizCharacterCommunicationService.get(targetId);
        Integer sourceSort = sourceEntity.getCharacterSort();
        Integer targetSort = targetEntity.getCharacterSort();


        //上移
        if (sourceSort > targetSort) {
            //target（包括自己）~ source的sort都+1
            //在原来id的基础上查找，因为并不需要移动id位置，只需要改变sort
            //查找
            String condition = "character_sort >= ? AND character_sort <= ?";
            Object[] paramValue = {targetSort, sourceSort};
            BizCharacterCommunicationEntity tempEntity1 = new BizCharacterCommunicationEntity();
            List<BizCharacterCommunicationEntity> resultModelList = bizCharacterCommunicationService.findByExample(tempEntity1, condition, paramValue);
            for (int i = 0; i < resultModelList.size(); i++) {
                if (resultModelList.get(i).getId().equals(sourceId)) {
                    continue;
                }
                Integer sort = resultModelList.get(i).getCharacterSort();
                resultModelList.get(i).setCharacterSort(sort + 1);
                //save
                bizCharacterCommunicationService.update(resultModelList.get(i));
            }
        } else { //下移
            //1.source（不包括自己）~ target（包括自己）t的sort都-1
            String condition = "character_sort >= ? AND character_sort <= ?";
            Object[] paramValue = {sourceSort, targetSort};
            BizCharacterCommunicationEntity tempEntity2 = new BizCharacterCommunicationEntity();
            List<BizCharacterCommunicationEntity> resultModelList = bizCharacterCommunicationService.findByExample(tempEntity2, condition, paramValue);
            for (int i = 0; i < resultModelList.size(); i++) {
                if (resultModelList.get(i).getId().equals(sourceId)) {
                    continue;
                }
                Integer sort = resultModelList.get(i).getCharacterSort();
                resultModelList.get(i).setCharacterSort(sort - 1);
                //save
                bizCharacterCommunicationService.update(resultModelList.get(i));
            }
        }
        //sourceEntity的sort值变为target的sort
        sourceEntity.setCharacterSort(targetSort);
        //save
        bizCharacterCommunicationService.update(sourceEntity);
    }

    @Override
    public SaveCharacterCommunicationEntity getCharacterCommunication(Integer saveCharacterCommunicationDtoId) throws Exception {
        SaveCharacterCommunicationEntity resultDto = new SaveCharacterCommunicationEntity();
        //判断角色对话是否存在
        BizCharacterCommunicationEntity bizCharacterCommunicationEntity = bizCharacterCommunicationService.get(saveCharacterCommunicationDtoId);
        if (bizCharacterCommunicationEntity == null || CommonConstant.IsDeleted.Y.equals(bizCharacterCommunicationEntity.getIsDeleted())) {
            return null;
        }
        //查出来的model放入返回值中
        resultDto = BizCharacterCommunicationConvert.entityToDtoEntity(bizCharacterCommunicationEntity);

        //判断theme是否存在
        BizCommunicationThemeEntity themeEntity = new BizCommunicationThemeEntity();
        themeEntity.setThemeCode(bizCharacterCommunicationEntity.getThemeCode());
        List<BizCommunicationThemeEntity> bizCommunicationThemeEntities = bizCommunicationThemeService.findByExample(themeEntity);
        //如果存在
        if (bizCommunicationThemeEntities.get(0) != null && CommonConstant.IsDeleted.N.equals(bizCommunicationThemeEntities.get(0).getIsDeleted())) {
            //查出来的model放入返回值
            CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
            BeanUtil.copyProperties(bizCommunicationThemeEntities.get(0), resultDto, copyOptions);
            //判断prompt是否存在
            BizAiPromptEntity promptEntity = new BizAiPromptEntity();
            promptEntity.setPromptCode(bizCommunicationThemeEntities.get(0).getPromptRelation());
            List<BizAiPromptEntity> bizAiPromptEntities = bizAiPromptService.findByExample(promptEntity);
            if (bizAiPromptEntities.get(0) != null && CommonConstant.IsDeleted.N.equals(bizAiPromptEntities.get(0).getIsDeleted())) {
                BeanUtil.copyProperties(bizAiPromptEntities.get(0), resultDto, copyOptions);
            }
        }
        return resultDto;
    }

    @Override
    public SaveCharacterCommunicationEntity save(SaveCharacterCommunicationEntity saveCharacterCommunicationEntity) throws Exception {
        //1.先创建prompt，写入db中
        BizAiPromptEntity promptEntity = new BizAiPromptEntity();
        promptEntity.setPromptValue(saveCharacterCommunicationEntity.getPromptValue()); //开场白（助手词）
        long promptCurrentTime = System.currentTimeMillis();
        long promptUniqueNumber = counter.incrementAndGet();
        // Combine current timestamp and unique number to form the unique code
        String promptCode = promptCurrentTime + "-" + promptUniqueNumber;
        promptEntity.setPromptCode(promptCode);     //设置promptCode
        bizAiPromptService.save(promptEntity);


        //2.再创建theme，其中 prompt_relation -> prompt_code，写入db中   里面包括角色设置、随机性设置、话题新鲜度
        BizCommunicationThemeEntity bizCommunicationThemeEntity = BizCharacterCommunicationConvert.dtoEntityToCommunicationThemeEntity(saveCharacterCommunicationEntity);
        bizCommunicationThemeEntity.setPromptRelation(promptCode);  //关联promptCode
        long themeCurrentTime = System.currentTimeMillis();
        long themeUniqueNumber = counter.incrementAndGet();
        // Combine current timestamp and unique number to form the unique code
        String themeCode = themeCurrentTime + "-" + themeUniqueNumber;
        bizCommunicationThemeEntity.setThemeCode(themeCode);        //设置themeCode
        bizCommunicationThemeService.save(bizCommunicationThemeEntity);

        //3.再创建characterCommunication，其中theme_code -> theme.theme_code，存入db
        //其中包括：角色名字、音色ID、角色描述、头像url
        BizCharacterCommunicationEntity bizCharacterCommunicationEntity = BizCharacterCommunicationConvert.dtoEntityToEntity(saveCharacterCommunicationEntity);
        bizCharacterCommunicationEntity.setThemeCode(themeCode);    //设置themeCode

        // 让新增的数据处于第一位，设置排序为 1 ，将后面所有数据的排序都 + 1
        bizCharacterCommunicationEntity.setCharacterSort(1);
        List<BizCharacterCommunicationEntity> communicationEntities = bizCharacterCommunicationService.getAll();
        for(BizCharacterCommunicationEntity communicationEntity : communicationEntities){
            communicationEntity.setCharacterSort(communicationEntity.getCharacterSort() + 1);
            bizCharacterCommunicationService.update(communicationEntity);
        }
        // 保存characterCommunication
        BizCharacterCommunicationEntity characterCommunicationEntity = bizCharacterCommunicationService.save(bizCharacterCommunicationEntity);

        //3.最后返回已经修改的saveCharacterCommunicationDto对象
        saveCharacterCommunicationEntity.setThemeCode(themeCode);
        saveCharacterCommunicationEntity.setPromptRelation(promptCode);
        saveCharacterCommunicationEntity.setCharacterSort(1);
        saveCharacterCommunicationEntity.setId(characterCommunicationEntity.getId());
        saveCharacterCommunicationEntity.setIsDeleted(characterCommunicationEntity.getIsDeleted());
        saveCharacterCommunicationEntity.setIsDisplay(characterCommunicationEntity.getIsDisplay());
        return saveCharacterCommunicationEntity;
    }

    @Override
    public SaveCharacterCommunicationEntity update(SaveCharacterCommunicationEntity saveCharacterCommunicationEntity) throws Exception {
        //判断角色对话是否存在
        BizCharacterCommunicationEntity bizCharacterCommunicationEntity = bizCharacterCommunicationService.get(saveCharacterCommunicationEntity.getId());
        if (bizCharacterCommunicationEntity == null || CommonConstant.IsDeleted.Y.equals(bizCharacterCommunicationEntity.getIsDeleted())) {
            throw new BusinessException("characterCommunicationId 为空");
        }
        //判断theme是否存在
        BizCommunicationThemeEntity themeEntity = bizCommunicationThemeService.getByThemeCode(bizCharacterCommunicationEntity.getThemeCode());
        if (themeEntity == null || CommonConstant.IsDeleted.Y.equals(themeEntity.getIsDeleted())) {
            throw new BusinessException("communicationThemeId 为空");
        }
        //判断prompt是否存在
        BizAiPromptEntity bizAiPromptEntity = bizAiPromptService.getByPromptCode(themeEntity.getPromptRelation());
        if (bizAiPromptEntity == null || CommonConstant.IsDeleted.Y.equals(bizAiPromptEntity.getIsDeleted())) {
            throw new BusinessException("promptId 为空");
        }

        //prompt更新
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(BizCharacterCommunicationConvert.dtoToPromptEntity(saveCharacterCommunicationEntity), bizAiPromptEntity, copyOptions);
        BizAiPromptEntity update = bizAiPromptService.update(bizAiPromptEntity);
        System.out.println("updateBizAiPrompt:"+update);
        //theme更新
        BeanUtil.copyProperties(BizCharacterCommunicationConvert.dtoEntityToCommunicationThemeEntity(saveCharacterCommunicationEntity), themeEntity, copyOptions);
        bizCommunicationThemeService.update(themeEntity);

        //characterCommunication更新
        BeanUtil.copyProperties(BizCharacterCommunicationConvert.dtoEntityToEntity(saveCharacterCommunicationEntity), bizCharacterCommunicationEntity, copyOptions);
        bizCharacterCommunicationService.update(bizCharacterCommunicationEntity);
        return saveCharacterCommunicationEntity;
    }

    @Override
    public void deleteCharacter(Integer bizCharacterCommunicationId) throws Exception {
        BizCharacterCommunicationEntity bizCharacterCommunicationEntity = bizCharacterCommunicationService.get(bizCharacterCommunicationId);
        //查找theme
        BizCommunicationThemeEntity communicationThemeEntity = bizCommunicationThemeService.getByThemeCode(bizCharacterCommunicationEntity.getThemeCode());
        Assert.notNull(communicationThemeEntity.getId(),"communicationThemeId 为空！");
        if (communicationThemeEntity != null) {
            //查找prompt并删除
            BizAiPromptEntity bizAiPromptEntity = bizAiPromptService.getByPromptCode(communicationThemeEntity.getPromptRelation());
            Assert.notNull(bizAiPromptEntity.getId(),"promptId 为空！");
            bizAiPromptService.deleteById(bizAiPromptEntity.getId());
        }
        Assert.notNull(communicationThemeEntity.getId(),"promptId 为空！");
        //删除theme
        bizCommunicationThemeService.deletedById(communicationThemeEntity.getId());
        //删除角色
        bizCharacterCommunicationService.deleteById(bizCharacterCommunicationId);
    }

    @Override
    public List<BizCharacterCommunicationEntity> getAllByAsc(PagingInfo pagingInfo) throws Exception {
        Assert.notNull(pagingInfo, "pagingInfo is null");
        BizCharacterCommunicationEntity entity = new BizCharacterCommunicationEntity();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        entity.setIsDisplay(CommonConstant.IsDeleted.Y);
        List<BizCharacterCommunicationEntity> entities = bizCharacterCommunicationService.findByExample(entity, "character_sort ASC", pagingInfo);
        return entities;
    }


}
