package cn.com.poc.communication.rest.impl;

import cn.com.poc.common.utils.Assert;
import cn.com.poc.communication.aggregation.CharacterCommunicationService;
import cn.com.poc.communication.convert.BizCharacterCommunicationConvert;
import cn.com.poc.communication.convert.SaveCharacterCommunicationDtoConvert;
import cn.com.poc.communication.dto.SaveCharacterCommunicationDto;
import cn.com.poc.communication.entity.BizCharacterCommunicationEntity;
import cn.com.poc.communication.rest.BizCharacterCommunicationRest;
import cn.com.poc.communication.service.BizCharacterCommunicationService;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.xfyun.exception.BusinessException;
import org.springframework.stereotype.Component;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Component
public class BizCharacterCommunicationRestImpl implements BizCharacterCommunicationRest {

    @Resource
    private BizCharacterCommunicationService bizCharacterCommunicationService;

    @Resource
    private CharacterCommunicationService characterCommunicationService;

    /**
     * 返回角色列表（分页列表）   //根据sort排序
     * */
    @Override
    public List<SaveCharacterCommunicationDto> getCharacterList(PagingInfo pagingInfo) throws Exception {
        Assert.notNull(pagingInfo,"参数不能为空");
        List<BizCharacterCommunicationEntity> characterCommunicationEntities = characterCommunicationService.getAllByAsc(pagingInfo);
        List<SaveCharacterCommunicationDto> resultList = new ArrayList<>();
        for(BizCharacterCommunicationEntity entity : characterCommunicationEntities){
            resultList.add(BizCharacterCommunicationConvert.entityToDto(entity));
        }
        return resultList;
    }

    /**
     * 更新角色排序
     * */
    @Override
    public void sortCharacter(Integer sourceId, Integer targetId) throws Exception {
        Assert.notNull(sourceId, "参数不能为空");
        Assert.notNull(targetId, "参数不能为空");
        characterCommunicationService.sortCharacter(sourceId, targetId);
    }

    /**
     * 作用在角色页面，是否显示
     * 前端传来一个角色，要将他设置为不显示
     * */
    @Override
    public void isDisplay(Integer bizCharacterCommunicationId) throws Exception {
        Assert.notNull(bizCharacterCommunicationId, "参数不能为空");
        bizCharacterCommunicationService.isDisplay(bizCharacterCommunicationId);
    }


    /**
     * 获取某一个角色会话配置
     * @return: SaveCharacterCommunicationDto 返回角色会话对象
     * 作用在角色会话页面
     * */
    @Override
    public SaveCharacterCommunicationDto getCharacterCommunication(Integer bizCharacterCommunicationId) throws Exception {
        Assert.notNull(bizCharacterCommunicationId, "参数不能为空");
        return SaveCharacterCommunicationDtoConvert.entityToDto(characterCommunicationService.getCharacterCommunication(bizCharacterCommunicationId));
    }

    /**
     * 保存角色
     * 作用在角色会话页面
     * */
    @Override
    public SaveCharacterCommunicationDto save(SaveCharacterCommunicationDto saveCharacterCommunicationDto) throws Exception {
        Assert.notNull(saveCharacterCommunicationDto, "CharacterCommunication is null");
        Assert.notNull(saveCharacterCommunicationDto.getCharacterName(), "CharacterCommunicationCharacterName is null");
        Assert.notNull(saveCharacterCommunicationDto.getPresencePenalty(), "CharacterCommunicationPresencePenalty is null");
        Assert.notNull(saveCharacterCommunicationDto.getTemperature(), "CharacterCommunicationTemperature is null");
        Assert.notNull(saveCharacterCommunicationDto.getAvatarUrl(), "CharacterCommunicationAvatarUrl is null");
        Assert.notNull(saveCharacterCommunicationDto.getTimbreId(), "CharacterCommunicationTimbreId is null");
        if(saveCharacterCommunicationDto.getId() != null){
            throw new BusinessException("id不能有值");
        }
        return SaveCharacterCommunicationDtoConvert.entityToDto(characterCommunicationService.save(SaveCharacterCommunicationDtoConvert.dtoToEntity(saveCharacterCommunicationDto)));
    }

    /**
     * 更新角色
     * 作用在角色会话页面
     * */
    @Override
    public SaveCharacterCommunicationDto update(SaveCharacterCommunicationDto saveCharacterCommunicationDto) throws Exception {
        Assert.notNull(saveCharacterCommunicationDto, "CharacterCommunication is null");
        Assert.notNull(saveCharacterCommunicationDto.getId(), "CharacterCommunicationId is null");
        return SaveCharacterCommunicationDtoConvert.entityToDto(characterCommunicationService.update(SaveCharacterCommunicationDtoConvert.dtoToEntity(saveCharacterCommunicationDto)));
    }

    /**
     * 删除角色
     * 作用在角色页面
     * */
    @Override
    public void deleteCharacter(Integer bizCharacterCommunicationId) throws Exception {
        Assert.notNull(bizCharacterCommunicationId, "参数不能为空");
        characterCommunicationService.deleteCharacter(bizCharacterCommunicationId);
    }
}
