package cn.com.poc.communication.rest.impl;

import cn.com.poc.communication.entity.BizCommunicationRankEntity;
import cn.com.poc.communication.rest.BizCommunicationRankRest;
import cn.com.poc.communication.result.CommunicationResult;
import cn.com.poc.communication.service.BizCommunicationRankService;
import cn.com.poc.conversational.entity.BizConversationalGradeWordsRelationEntity;
import cn.com.poc.conversational.service.BizConversationalGradeWordsRelationService;
import cn.com.yict.framemax.data.model.PagingInfo;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author Roger Wu
 */
@Component
class BizCommunicationRankRestImpl implements BizCommunicationRankRest {

    @Resource
    private BizConversationalGradeWordsRelationService bizConversationalGradeWordsRelationService;


    @Resource
    private BizCommunicationRankService bizCommunicationRankService;

    /**
     * 创建对话等级
     *
     * @param bizCommunicationRankEntity
     * @throws Exception
     */
    @Override
    public void createCommunicateRank(BizCommunicationRankEntity bizCommunicationRankEntity) throws Exception {
        bizCommunicationRankService.createCommunicateRank(bizCommunicationRankEntity);
    }


    /**
     * 更新对话等级
     *
     * @param bizCommunicationRankEntity
     * @throws Exception
     */
    @Override
    public void updateCommunicateRank(BizCommunicationRankEntity bizCommunicationRankEntity) throws Exception {
        bizCommunicationRankService.updateCommunicateRank(bizCommunicationRankEntity);
    }


    /**
     * 删除对话等级
     *
     * @param communicationRankId
     * @throws Exception
     */
    @Override
    public void removeCommunicateRank(Integer communicationRankId) throws Exception {
        bizCommunicationRankService.removeById(communicationRankId);
    }

    @Override
    public List<CommunicationResult> communicationRankList(BizCommunicationRankEntity entity, PagingInfo pagingInfo) throws Exception {
        List<BizCommunicationRankEntity> bizCommunicationRankEntityList = bizCommunicationRankService.findByExample(entity,pagingInfo);

        List<String> difficultyGradeList = bizCommunicationRankEntityList.stream().map(item -> item.getDifficultyGrade()).distinct().collect(Collectors.toList());

        List<CommunicationResult> communicationResultList = new ArrayList<>();
        for (BizCommunicationRankEntity bizCommunicationRankEntity : bizCommunicationRankEntityList) {
            CommunicationResult communicationResult = new CommunicationResult();
            communicationResult.setCommunicationRankId(bizCommunicationRankEntity.getCommunicationRankId());
            communicationResult.setDifficultyGrade(bizCommunicationRankEntity.getDifficultyGrade());
            communicationResult.setDifficultyText(bizCommunicationRankEntity.getDifficultyText());
            communicationResultList.add(communicationResult);
        }

        if (CollectionUtils.isNotEmpty(difficultyGradeList)) {
            BizConversationalGradeWordsRelationEntity conversationalGradeWordsRelationEntity = new BizConversationalGradeWordsRelationEntity();
            conversationalGradeWordsRelationEntity.setDifficultyGradeList(difficultyGradeList);
            List<BizConversationalGradeWordsRelationEntity> bizConversationalGradeWordsRelationEntities = bizConversationalGradeWordsRelationService.conversationalGradeWordsRelationList(conversationalGradeWordsRelationEntity);
            Map<String, List<BizConversationalGradeWordsRelationEntity>> conversationalGradeWordsRelationEntityMap = bizConversationalGradeWordsRelationEntities.stream().collect(Collectors.groupingBy(item -> item.getDifficultyGrade()));
            for (CommunicationResult communicationResult : communicationResultList) {
                if (MapUtils.isNotEmpty(conversationalGradeWordsRelationEntityMap)) {
                    List<BizConversationalGradeWordsRelationEntity> conversationalGradeWordsRelationEntities = conversationalGradeWordsRelationEntityMap.get(communicationResult.getDifficultyGrade());
                    if (CollectionUtils.isNotEmpty(conversationalGradeWordsRelationEntities)) {
                        BizConversationalGradeWordsRelationEntity bizConversationalGradeWordsRelationEntity = conversationalGradeWordsRelationEntities.get(0);
                        communicationResult.setVocabularys(bizConversationalGradeWordsRelationEntity.getWordsSource());
                    }
                }
            }
        }
        return communicationResultList;
    }
}