package cn.com.poc.communication.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.communication.convert.BizAiPromptConvert;
import cn.com.poc.communication.entity.BizAiPromptEntity;
import cn.com.poc.communication.model.BizAiPromptModel;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.poc.communication.service.BizAiPromptService;
import cn.com.poc.communication.repository.BizAiPromptRepository;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;

@Service
public class BizAiPromptServiceImpl extends BaseServiceImpl
        implements BizAiPromptService {

    @Resource
    private BizAiPromptRepository repository;


    @Override
    public BizAiPromptEntity get(Integer id) {
        Assert.notNull(id, "promptId is null");
        BizAiPromptModel model = this.repository.get(id);
        if (model == null || model.getIsDeleted().equals(CommonConstant.IsDeleted.Y)) {
            return null;
        }
        return BizAiPromptConvert.modelToEntity(model);
    }

    @Override
    public BizAiPromptEntity getByPromptCode(String promptCode) {
        Assert.notBlank(promptCode);
        BizAiPromptModel model = new BizAiPromptModel();
        model.setPromptCode(promptCode);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizAiPromptModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            BizAiPromptEntity entity = new BizAiPromptEntity();
            BeanUtil.copyProperties(models.get(0), entity);
            return entity;
        }
        return null;
    }

    @Override
    public List<BizAiPromptEntity> findByExample(BizAiPromptEntity entity) {
        List<BizAiPromptEntity> result = new ArrayList<>();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizAiPromptModel model = BizAiPromptConvert.entityToModel(entity);
        List<BizAiPromptModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(BizAiPromptConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public void deleteById(Integer id) {
        Assert.notNull(id, "promptId is null");
        BizAiPromptModel model = this.repository.get(id);
        if (null == model) {
            return;
        }
        model.setIsDeleted(CommonConstant.IsDeleted.Y);
        repository.save(model);
    }

    @Override
    public BizAiPromptEntity save(BizAiPromptEntity entity) {
        Assert.notNull(entity, "entity is null");
        entity.setId(null);
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizAiPromptModel model = BizAiPromptConvert.entityToModel(entity);
        BizAiPromptModel saveModel = this.repository.save(model);
        return BizAiPromptConvert.modelToEntity(saveModel);
    }

    @Override
    public BizAiPromptEntity update(BizAiPromptEntity entity) {
        Assert.notNull(entity, "entity is null");
        Assert.notNull(entity.getId(), "bizAiPromptEntityId is null");
        BizAiPromptModel promptModel = this.repository.get(entity.getId());
        if (promptModel == null || CommonConstant.IsDeleted.Y.equals(promptModel.getIsDeleted())) {
            return null;
        }
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(entity, promptModel, copyOptions);
        BizAiPromptModel saveModel = this.repository.save(promptModel);
        return BizAiPromptConvert.modelToEntity(saveModel);
    }
}