package cn.com.poc.communication.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.communication.convert.BizCharacterCommunicationConvert;
import cn.com.poc.communication.entity.BizCharacterCommunicationEntity;
import cn.com.poc.communication.model.BizCharacterCommunicationModel;
import cn.com.poc.communication.repository.BizCharacterCommunicationRepository;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.poc.communication.service.BizCharacterCommunicationService;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;

@Service
public class BizCharacterCommunicationServiceImpl extends BaseServiceImpl
        implements BizCharacterCommunicationService {

    @Resource
    private BizCharacterCommunicationRepository repository;
    @Override
    public BizCharacterCommunicationEntity get(Integer id) throws Exception {
        Assert.notNull(id, "characterCommunicationId is null");
        BizCharacterCommunicationModel model = this.repository.get(id);
        if (model == null || model.getIsDeleted().equals(CommonConstant.IsDeleted.Y)) {
            return null;
        }
        return BizCharacterCommunicationConvert.modelToEntity(model);
    }


    @Override
    public List<BizCharacterCommunicationEntity> getAll() throws Exception {
        List<BizCharacterCommunicationEntity> result = new ArrayList<>();
        List<BizCharacterCommunicationModel> models = this.repository.getAll();
        for (BizCharacterCommunicationModel model : models) {
            result.add(BizCharacterCommunicationConvert.modelToEntity(model));

        }
        return result;
    }

    @Override
    public List<BizCharacterCommunicationEntity> findByExample(BizCharacterCommunicationEntity entity, PagingInfo pagingInfo) throws Exception {
        List<BizCharacterCommunicationEntity> result = new ArrayList<>();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizCharacterCommunicationModel model = BizCharacterCommunicationConvert.entityToModel(entity);
        List<BizCharacterCommunicationModel> models = this.repository.findByExample(model, pagingInfo);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(BizCharacterCommunicationConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public List<BizCharacterCommunicationEntity> findByExample(BizCharacterCommunicationEntity entity, String condition, PagingInfo pagingInfo) throws Exception {
        List<BizCharacterCommunicationEntity> result = new ArrayList<>();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizCharacterCommunicationModel model = BizCharacterCommunicationConvert.entityToModel(entity);
        List<BizCharacterCommunicationModel> models = this.repository.findByExample(model, condition, pagingInfo);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(BizCharacterCommunicationConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public List<BizCharacterCommunicationEntity> findByExample(BizCharacterCommunicationEntity entity, String condition, Object[] paramValue) throws Exception {
        List<BizCharacterCommunicationEntity> result = new ArrayList<>();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizCharacterCommunicationModel model = BizCharacterCommunicationConvert.entityToModel(entity);
        List<BizCharacterCommunicationModel> models = this.repository.findByExample(model, condition, paramValue);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(BizCharacterCommunicationConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }


    @Override
    public void deleteById(Integer id) throws Exception {
        Assert.notNull(id, "characterCommunicationId is null");
        //查找characterCommunication
        BizCharacterCommunicationModel characterCommunicationModel = this.repository.get(id);
        if (null == characterCommunicationModel) {
            return;
        }
        characterCommunicationModel.setIsDeleted(CommonConstant.IsDeleted.Y);
        repository.save(characterCommunicationModel);
    }

    @Override
    public BizCharacterCommunicationEntity save(BizCharacterCommunicationEntity entity) throws Exception {
        Assert.notNull(entity, "Cannot save empty objects");
        entity.setId(null);
        BizCharacterCommunicationModel bizCharacterCommunicationModel = BizCharacterCommunicationConvert.entityToModel(entity);
        BizCharacterCommunicationModel model = this.repository.save(bizCharacterCommunicationModel);
        return BizCharacterCommunicationConvert.modelToEntity(model);
    }

    @Override
    public BizCharacterCommunicationEntity update(BizCharacterCommunicationEntity entity) throws Exception {
        Assert.notNull(entity, "entity is null");
        Assert.notNull(entity.getId(), "bizCharacterCommunicationId is null");
        BizCharacterCommunicationModel characterCommunicationModel = this.repository.get(entity.getId());
        if (characterCommunicationModel == null || CommonConstant.IsDeleted.Y.equals(characterCommunicationModel.getIsDeleted())) {
            return null;
        }
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(entity, characterCommunicationModel, copyOptions);
        BizCharacterCommunicationModel saveModel = this.repository.save(characterCommunicationModel);
        return BizCharacterCommunicationConvert.modelToEntity(saveModel);
    }


    @Override
    public void isDisplay(Integer bizCharacterCommunicationId) throws Exception {
        Assert.notNull(bizCharacterCommunicationId, "characterCommunicationId is null");
        BizCharacterCommunicationModel model = this.repository.get(bizCharacterCommunicationId);
        BizCharacterCommunicationEntity entity = BizCharacterCommunicationConvert.modelToEntity(model);
        if(CommonConstant.IsDeleted.Y.equals(entity.getIsDisplay())){
            entity.setIsDisplay(CommonConstant.IsDeleted.N);
        }else {
            entity.setIsDisplay(CommonConstant.IsDeleted.Y);
        }
        //更新
        this.update(entity);
    }
}