package cn.com.poc.communication.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.entity.BatchInsertResult;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.common.utils.BlContext;
import cn.com.poc.common.utils.SQLUtils;
import cn.com.poc.communication.convert.BizCommunicationGradeWordsConvert;
import cn.com.poc.communication.entity.BizCommunicationGradeWordsEntity;
import cn.com.poc.communication.model.BizCommunicationGradeWordsModel;
import cn.com.poc.communication.query.SearchGradeWordsQueryCondition;
import cn.com.poc.communication.query.SearchGradeWordsQueryItem;
import cn.com.poc.communication.repository.BizCommunicationGradeWordsRepository;
import cn.com.poc.communication.service.BizCommunicationGradeWordsService;
import cn.com.poc.support.security.oauth.entity.UserBaseEntity;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.yict.framemax.data.model.PagingInfo;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class BizCommunicationGradeWordsServiceImpl extends BaseServiceImpl
        implements BizCommunicationGradeWordsService {

    final private Logger logger = LoggerFactory.getLogger(BizCommunicationGradeWordsService.class);

    @Resource
    private BizCommunicationGradeWordsRepository repository;

    @Override
    public BizCommunicationGradeWordsEntity save(BizCommunicationGradeWordsEntity entity) {
        Assert.notNull(entity, "Cannot save empty objects");
        Assert.notBlank(entity.getWord(), "word cannot be blank");
        Assert.notNull(entity.getTokenId(), "token id is null");
        Assert.notBlank(entity.getDifficultyGrade(), "grade cannot be blank");

        BizCommunicationGradeWordsModel model = new BizCommunicationGradeWordsModel();
        model.setWord(entity.getWord());
        model.setTokenId(entity.getTokenId());
        model.setDifficultyGrade(entity.getDifficultyGrade());
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizCommunicationGradeWordsModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            logger.warn("The word exists");
            return null;
        }
        BizCommunicationGradeWordsModel wordsModel = this.repository.save(model);
        return BizCommunicationGradeWordsConvert.modelToEntity(wordsModel);
    }

    @Override
    public void deleted(BizCommunicationGradeWordsEntity entity) {
        Assert.notNull(entity, "entity can not be null");
        BizCommunicationGradeWordsModel model = new BizCommunicationGradeWordsModel();
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        model.setWord(entity.getWord());
        model.setDifficultyGrade(entity.getDifficultyGrade());
        List<BizCommunicationGradeWordsModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            BizCommunicationGradeWordsModel wordsModel = models.get(0);
            wordsModel.setIsDeleted(CommonConstant.IsDeleted.Y);
            this.repository.save(wordsModel);
        }
    }

    @Override
    public void batchSave(List<BizCommunicationGradeWordsEntity> entities) throws Exception {
        Assert.notEmpty(entities, "Cannot save empty objects");
        UserBaseEntity currentUser = BlContext.getCurrentUserNotException();
        List<BizCommunicationGradeWordsModel> models = entities.stream()
                .map(entity -> {
                            Date now = new Date();
                            BizCommunicationGradeWordsModel model = BizCommunicationGradeWordsConvert.entityToModel(entity);
                            model.setIsDeleted(CommonConstant.IsDeleted.N);
                            model.setCreator(currentUser == null ? CommonConstant.SYSTEM_FIELD.SYSTEM : currentUser.getUserName());
                            model.setCreatedTime(now);
                            model.setModifier(currentUser == null ? CommonConstant.SYSTEM_FIELD.SYSTEM : currentUser.getUserName());
                            model.setModifiedTime(now);
                            model.setSysVersion(0);
                            return model;
                        }
                ).collect(Collectors.toList());
        BatchInsertResult batchInsertResult = SQLUtils.getInsertSql(BizCommunicationGradeWordsModel.class, models);
        this.sqlDao.batchUpdate(batchInsertResult.getInsertSQL(), batchInsertResult.getInsertParams().toArray(new Object[0][]));
    }

    @Override
    public List<BizCommunicationGradeWordsEntity> findByExample(BizCommunicationGradeWordsEntity entity) throws Exception {
        return this.findByExample(entity, null);
    }

    @Override
    public List<BizCommunicationGradeWordsEntity> findByExample(BizCommunicationGradeWordsEntity entity, PagingInfo pagingInfo) throws Exception {
        List<BizCommunicationGradeWordsEntity> result = new ArrayList<>();
        BizCommunicationGradeWordsModel model = BizCommunicationGradeWordsConvert.entityToModel(entity);
        List<BizCommunicationGradeWordsModel> models = this.repository.findByExample(model, pagingInfo);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(BizCommunicationGradeWordsConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public List<SearchGradeWordsQueryItem> searchWords(SearchGradeWordsQueryCondition condition, PagingInfo pagingInfo) throws Exception {
        return this.sqlDao.query(condition, SearchGradeWordsQueryItem.class, pagingInfo);
    }

    @Override
    public void removeRepeated() {
        String sql = "update biz_communication_grade_words as t1 left join (select MIN(id) as min_id from biz_communication_grade_words group by word, difficulty_grade) as t2 on t1.id = t2.min_id set t1.is_deleted = 'Y' where t2.min_id is null;";
        this.sqlDao.executeDDL(sql);
    }

    @Override
    public void deletedWords(List<String> words) throws Exception {
        Assert.notEmpty(words, "words can not be empty");
        for (String word : words) {
            BizCommunicationGradeWordsModel model = new BizCommunicationGradeWordsModel();
            model.setWord(word);
            model.setIsDeleted(CommonConstant.IsDeleted.N);
            List<BizCommunicationGradeWordsModel> models = this.repository.findByExample(model);
            if (CollectionUtils.isNotEmpty(models)) {
                for (BizCommunicationGradeWordsModel wordsModel : models) {
                    wordsModel.setIsDeleted(CommonConstant.IsDeleted.Y);
                    this.repository.save(wordsModel);
                }
            }
        }
    }

    @Override
    public void deletedByGrade(String grade) {
        String sql = "UPDATE biz_communication_grade_words SET is_deleted='Y' WHERE difficulty_grade=?;";
        Object[] param = new Object[]{grade};
        this.sqlDao.batchUpdate(sql, new Object[][]{param});
    }
}