package cn.com.poc.communication.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.communication.convert.BizCommunicationRankConvert;
import cn.com.poc.communication.entity.BizCommunicationEntity;
import cn.com.poc.communication.entity.BizCommunicationRankEntity;
import cn.com.poc.communication.model.BizCommunicationRankModel;
import cn.com.poc.communication.repository.BizCommunicationRankRepository;
import cn.com.poc.communication.service.BizCommunicationGradeWordsService;
import cn.com.poc.communication.service.BizCommunicationRankService;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.com.yict.framemax.data.model.PagingInfo;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Roger Wu
 */
@Component
public class BizCommunicationRankServiceImpl implements BizCommunicationRankService {


    @Resource
    private BizCommunicationRankRepository repository;

    @Resource
    private BizCommunicationGradeWordsService bizCommunicationGradeWordsService;

    @Override
    public void createCommunicateRank(BizCommunicationRankEntity bizCommunicationRankEntity) {
        Assert.notNull(bizCommunicationRankEntity.getDifficultyGrade(), "请输入口语等级代码");
        Assert.notNull(bizCommunicationRankEntity.getDifficultyText(), "请输入口语等级对话标准");

        BizCommunicationRankModel example = new BizCommunicationRankModel();
        example.setDifficultyGrade(bizCommunicationRankEntity.getDifficultyGrade());
        example.setIsDeleted(CommonConstant.IsDeleted.N);
        if (CollectionUtils.isNotEmpty(repository.findByExample(example))) {
            throw new BusinessException("等级代码：\'" + bizCommunicationRankEntity.getDifficultyGrade() + "\'已存在,请重新输入!");
        }

        BizCommunicationRankModel model = new BizCommunicationRankModel();
        model.setDifficultyGrade(bizCommunicationRankEntity.getDifficultyGrade());
        model.setDifficultyText(bizCommunicationRankEntity.getDifficultyText());
        repository.save(model);
    }

    @Override
    public void updateCommunicateRank(BizCommunicationRankEntity bizCommunicationRankEntity) {
        Assert.notNull(bizCommunicationRankEntity.getCommunicationRankId(), "口语等级主键不能为空");
//        Assert.notNull(bizCommunicationRankEntity.getDifficultyGrade(), "请输入口语等级代码");
        Assert.notNull(bizCommunicationRankEntity.getDifficultyText(), "请输入口语等级对话标准");

        BizCommunicationRankModel bizCommunicationRankModel = repository.get(bizCommunicationRankEntity.getCommunicationRankId());
        if (bizCommunicationRankModel == null || CommonConstant.IsDeleted.Y.equals(bizCommunicationRankModel.getIsDeleted())) {
            throw new BusinessException("根据该主键找不到口语等级信息");
        }

        BizCommunicationRankModel example = new BizCommunicationRankModel();
        example.setDifficultyGrade(bizCommunicationRankEntity.getDifficultyGrade());
        example.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizCommunicationRankModel> byExampleList = repository.findByExample(example);
        if (CollectionUtils.isNotEmpty(byExampleList)) {
            List<Integer> communicatioRankIdList = byExampleList.stream().map(item -> item.getCommunicationRankId()).collect(Collectors.toList());
            if (!communicatioRankIdList.contains(bizCommunicationRankEntity.getCommunicationRankId())) {
                throw new BusinessException("等级代码：\'" + bizCommunicationRankEntity.getDifficultyGrade() + "\'已存在,请重新输入!");
            }
        }

//      bizCommunicationRankModel.setDifficultyGrade(bizCommunicationRankEntity.getDifficultyGrade());
        bizCommunicationRankModel.setDifficultyText(bizCommunicationRankEntity.getDifficultyText());
        repository.save(bizCommunicationRankModel);
    }

    @Override
    public void removeById(Integer communicationRankId) throws Exception {
        Assert.notNull(communicationRankId, "删除的主键id不能为空");
        BizCommunicationRankModel bizCommunicationRankModel = repository.get(communicationRankId);
        if (bizCommunicationRankModel == null || CommonConstant.IsDeleted.Y.equals(bizCommunicationRankModel.getIsDeleted())) {
            throw new BusinessException("根据该主键找不到口语等级信息");
        }
        bizCommunicationRankModel.setIsDeleted(CommonConstant.IsDeleted.Y);
        repository.save(bizCommunicationRankModel);

        //删除等级附带的单词
        bizCommunicationGradeWordsService.deletedByGrade(bizCommunicationRankModel.getDifficultyGrade());

    }


    @Override
    public List<BizCommunicationRankEntity> findByExample(BizCommunicationRankEntity entity, PagingInfo pagingInfo) throws Exception {
        BizCommunicationRankModel model = BizCommunicationRankConvert.entityToModel(entity);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizCommunicationRankModel> bizCommunicationRankModelList = repository.findByExample(model, pagingInfo);
        List<BizCommunicationRankEntity> bizCommunicationRankEntityList = new ArrayList<BizCommunicationRankEntity>();
        if (CollectionUtils.isNotEmpty(bizCommunicationRankModelList)) {
            bizCommunicationRankEntityList = bizCommunicationRankModelList.stream().map(BizCommunicationRankConvert::modelToEntity).collect(Collectors.toList());
        }
        return bizCommunicationRankEntityList;
    }

    @Override
    public BizCommunicationEntity save(BizCommunicationEntity entity) throws Exception {


        return null;
    }

    @Override
    public Boolean isExistsByDifficultyGrade(String difficulty) throws Exception {
        Assert.notBlank(difficulty, "difficulty can not empty");
        BizCommunicationRankModel model = new BizCommunicationRankModel();
        model.setDifficultyGrade(difficulty);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizCommunicationRankModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            return true;
        }
        return false;
    }

    @Override
    public BizCommunicationRankEntity getByGrade(String difficultyGrade) throws Exception {
        Assert.notBlank(difficultyGrade, "difficultyGrade can not empty");
        BizCommunicationRankModel model = new BizCommunicationRankModel();
        model.setDifficultyGrade(difficultyGrade);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizCommunicationRankModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            BizCommunicationRankModel bizCommunicationRankModel = models.get(0);
            return BizCommunicationRankConvert.modelToEntity(bizCommunicationRankModel);
        }
        return null;
    }

    @Override
    public List<String> getGrades(){
        List<String> result = new ArrayList<>();
        BizCommunicationRankModel model = new BizCommunicationRankModel();
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizCommunicationRankModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(BizCommunicationRankModel::getDifficultyGrade).collect(Collectors.toList());
        }
        return result;
    }
}
