package cn.com.poc.communication.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.communication.convert.BizCommunicationThemeConvert;
import cn.com.poc.communication.entity.BizCommunicationThemeEntity;
import cn.com.poc.communication.model.BizCommunicationThemeModel;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.poc.communication.service.BizCommunicationThemeService;
import cn.com.poc.communication.repository.BizCommunicationThemeRepository;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

@Service
public class BizCommunicationThemeServiceImpl extends BaseServiceImpl
        implements BizCommunicationThemeService {

    @Resource
    private BizCommunicationThemeRepository repository;

    public BizCommunicationThemeEntity getByThemeCode(String themeCode) {
        BizCommunicationThemeModel model = new BizCommunicationThemeModel();
        model.setThemeCode(themeCode);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizCommunicationThemeModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            BizCommunicationThemeEntity entity = new BizCommunicationThemeEntity();
            BeanUtil.copyProperties(models.get(0), entity);
            return entity;
        }
        return null;
    }

    @Override
    public boolean themeCodeIsExists(String themeCode) {
        Assert.notBlank(themeCode);
        BizCommunicationThemeModel model = new BizCommunicationThemeModel();
        model.setThemeCode(themeCode);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizCommunicationThemeModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            return true;
        }
        return false;
    }

    @Override
    public BizCommunicationThemeEntity randomTheme() {
        BizCommunicationThemeModel model = new BizCommunicationThemeModel();
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizCommunicationThemeModel> models = this.repository.findByExample(model);
        return models.stream().skip(new Random().nextInt(models.size())).map(value->{
            BizCommunicationThemeEntity entity = new BizCommunicationThemeEntity();
            BeanUtil.copyProperties(value, entity);
            return entity;
        }).findFirst().orElse(null);
    }

    @Override
    public BizCommunicationThemeEntity save(BizCommunicationThemeEntity entity) {
        Assert.notNull(entity, "entity is null");
        entity.setId(null);
        BizCommunicationThemeModel bizCommunicationThemeModel = BizCommunicationThemeConvert.entityToModel(entity);
        BizCommunicationThemeModel saveModel = this.repository.save(bizCommunicationThemeModel);
        return BizCommunicationThemeConvert.modelToEntity(saveModel);
    }

    @Override
    public void deletedById (Integer id) {
        Assert.notNull(id, "promptId is null");
        BizCommunicationThemeModel bizCommunicationThemeModel = this.repository.get(id);
        if (null == bizCommunicationThemeModel) {
            return;
        }
        bizCommunicationThemeModel.setIsDeleted(CommonConstant.IsDeleted.Y);
        repository.save(bizCommunicationThemeModel);
    }
    @Override
    public List<BizCommunicationThemeEntity> findByExample(BizCommunicationThemeEntity entity) {
        List<BizCommunicationThemeEntity> result = new ArrayList<>();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizCommunicationThemeModel model = BizCommunicationThemeConvert.entityToModel(entity);
        List<BizCommunicationThemeModel> models = this.repository.findByExample(model);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(BizCommunicationThemeConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public BizCommunicationThemeEntity update(BizCommunicationThemeEntity entity) {
        Assert.notNull(entity, "entity is null");
        Assert.notNull(entity.getId(), "themeId is null");
        BizCommunicationThemeModel themeModel = this.repository.get(entity.getId());
        if (themeModel == null || CommonConstant.IsDeleted.Y.equals(themeModel.getIsDeleted())) {
            return null;
        }
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(entity, themeModel, copyOptions);
        BizCommunicationThemeModel saveModel = this.repository.save(themeModel);
        return BizCommunicationThemeConvert.modelToEntity(saveModel);
    }
}