package cn.com.poc.expose.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.common.utils.UUIDTool;
import cn.com.poc.communication.entity.BizCommunicationEntity;
import cn.com.poc.communication.entity.BizCommunicationRecordEntity;
import cn.com.poc.communication.entity.BizCommunicationThemeEntity;
import cn.com.poc.communication.service.BizCommunicationRecordService;
import cn.com.poc.communication.service.BizCommunicationService;
import cn.com.poc.communication.service.BizCommunicationThemeService;
import cn.com.poc.expose.dto.GetCommunicationContextResult;
import cn.com.poc.expose.dto.InitCommunicationResponse;
import cn.com.poc.expose.dto.InitCommunicationResult;
import cn.com.poc.expose.dto.TextToAudioResponse;
import cn.com.poc.expose.service.CommunicationService;
import cn.com.poc.thirdparty.resource.tencent.service.TencentService;
import com.github.houbb.opencc4j.util.ZhConverterUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author alex.yao
 * @date 2023/11/23
 **/
@Service
public class CommunicationServiceImpl implements CommunicationService {


    @Resource
    private BizCommunicationThemeService bizCommunicationThemeService;

    @Resource
    private BizCommunicationRecordService bizCommunicationRecordService;

    @Resource
    private BizCommunicationService bizCommunicationService;

    @Resource
    private TencentService tencentService;

    @Override
    public InitCommunicationResult initCommunication(InitCommunicationResponse response) throws Exception {


        // 会话ID
        String communicationId = "COMM-" + UUIDTool.getUUID();
        // 创建会话配置
        BizCommunicationEntity bizCommunicationEntity = new BizCommunicationEntity();
        bizCommunicationEntity.setCommunicationId(communicationId);
        bizCommunicationEntity.setThemeCode(response.getThemeCode());
        bizCommunicationEntity.setLanguage(StringUtils.isNotBlank(response.getLanguage()) ? response.getLanguage() : "English");
        bizCommunicationEntity.setStandardCode(null);
        bizCommunicationEntity.setIsDeleted(CommonConstant.IsDeleted.N);
        bizCommunicationService.save(bizCommunicationEntity);

        //获取引导语
        BizCommunicationThemeEntity themeEntity = bizCommunicationThemeService.getByThemeCode(response.getThemeCode());

        //构建出参
        InitCommunicationResult result = new InitCommunicationResult();
        result.setCommunicationId(communicationId);
        result.setIntroduction(themeEntity.getAssistantContent());
        result.setIntroductionTranslation(ZhConverterUtil.convertToTraditional(themeEntity.getAssistantContent()));
        return result;
    }

    @Override
    public String textToAudio(TextToAudioResponse response) throws Exception {
        return tencentService.textToAudio(response.getText(), response.getVoiceId());
    }

    @Override
    public List<GetCommunicationContextResult> getCommunicationContext(String communicationId) throws Exception {
        Assert.notBlank(communicationId, "communication id can not be empty");
        List<GetCommunicationContextResult> result = new ArrayList<>();
        List<BizCommunicationRecordEntity> bizCommunicationRecordEntities = bizCommunicationRecordService.get(communicationId);
        if (CollectionUtils.isEmpty(bizCommunicationRecordEntities)) {
            return result;
        }
        for (BizCommunicationRecordEntity bizCommunicationRecordEntity : bizCommunicationRecordEntities) {
            GetCommunicationContextResult getCommunicationContextResult = new GetCommunicationContextResult();
            getCommunicationContextResult.setRole(bizCommunicationRecordEntity.getRole());
            getCommunicationContextResult.setContext(bizCommunicationRecordEntity.getContent());
            getCommunicationContextResult.setTransContent(bizCommunicationRecordEntity.getTransContent());
            getCommunicationContextResult.setDatetime(bizCommunicationRecordEntity.getTimestamp());
            getCommunicationContextResult.setVoiceUrl(bizCommunicationRecordEntity.getSpeechUrl());
            result.add(getCommunicationContextResult);
        }
        return result;
    }
}

