package cn.com.poc.expose.websocket.handler;


import cn.com.poc.thirdparty.resource.tencent.EngineModelTypeEnum;
import cn.com.poc.thirdparty.resource.tencent.ecognition.SpeechRecognizerBuilder;
import cn.com.poc.thirdparty.resource.tencent.listener.SpeechToTextListener;
import cn.com.poc.common.utils.SpringUtils;
import cn.com.poc.common.utils.StringUtils;
import cn.com.poc.expose.dto.CommunicationWsResponse;
import cn.com.yict.framemax.frame.service.FmxParamConfigService;
import com.alibaba.fastjson.JSONObject;
import com.tencent.asr.service.SpeechRecognizer;
import org.java_websocket.WebSocket;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

public class SpeechToTextHandler extends AbstractWsHandler {
    private final Logger logger = LoggerFactory.getLogger(SpeechToTextHandler.class);

    private boolean isInit;

    private boolean recognizerStart;

    private int needvad;

    private boolean isSpeechRecognizerStop;

    private SpeechRecognizer speechRecognizer;

    private SpeechToTextListener speechToTextListener;

    private List<byte[]> userAudioByteList = new ArrayList<>();


    @Override
    public void doHandler(WebSocket webSocket, String s) {
        CommunicationWsResponse communicationWsResponse = JSONObject.parseObject(s, CommunicationWsResponse.class);

        parse(webSocket, communicationWsResponse);

        end(communicationWsResponse);
    }

    /**
     * 解析音频数据
     *
     * @param communicationWsResponse
     */
    private void parse(WebSocket webSocket, CommunicationWsResponse communicationWsResponse) {
        createSpeechRecognizer(webSocket, communicationWsResponse);
        if (StringUtils.isNotBlank(communicationWsResponse.getUserAudioBase64()) && speechRecognizer != null) {
            //获取音频二进制
            byte[] bytes = Base64.getDecoder().decode(communicationWsResponse.getUserAudioBase64());
            //拼接用户语音
            userAudioByteList.add(bytes);
            //写入STT
            speechRecognizer.write(bytes);
        }
    }

    /**
     * 判断是否结束
     *
     * @param communicationWsResponse
     */
    private void end(CommunicationWsResponse communicationWsResponse) {
        String IS_FINISH = "finish";
        if (IS_FINISH.equals(communicationWsResponse.getIsFinish()) || (speechToTextListener != null && speechToTextListener.getIsFinal().get())) {
            if (speechRecognizer != null && !isSpeechRecognizerStop) {
                speechRecognizer.stop();
                isSpeechRecognizerStop = true;
                logger.info("speech recognizer stop");
            }
        }
    }

    /**
     * 创建STT客户端
     *
     * @throws Exception
     */
    private void createSpeechRecognizer(WebSocket webSocket, CommunicationWsResponse communicationWsResponse) {
        init(webSocket, communicationWsResponse);
        if (!recognizerStart) {
            FmxParamConfigService fmxParamConfigService = SpringUtils.getBean(FmxParamConfigService.class);
            String modelType = fmxParamConfigService.getParam("communication.engine.model");
            SpeechRecognizerBuilder speechRecognizerBuilder = new SpeechRecognizerBuilder()
                    .speechRecognitionListener(this.speechToTextListener)
                    .needvad(needvad)
                    .voiceFormat(12)
                    .noiseThreshold(1F)
                    .engineModelType(StringUtils.isNotBlank(modelType) ? modelType : EngineModelTypeEnum.en_US.getEngineModelType());
            if (needvad == 1) {
                speechRecognizerBuilder.maxSpeakTime(5000);
            }
            speechRecognizer = speechRecognizerBuilder.build();
            recognizerStart = speechRecognizer.start();
        }
    }

    /**
     * 初始化stt监听器
     *
     * @param webSocket
     */
    private void init(WebSocket webSocket, CommunicationWsResponse communicationWsResponse) {
        if (this.isInit) {
            return;
        }
        this.speechToTextListener = new SpeechToTextListener();
        this.speechToTextListener.setWebSocket(webSocket);
        this.speechToTextListener.setUserAudioByteList(this.userAudioByteList);
        this.isInit = true;
        this.needvad = communicationWsResponse.getNeedvAD() != null ? communicationWsResponse.getNeedvAD() : 0;
    }
}
