package cn.com.poc.message.service.impl;


import cn.com.poc.audio_generation.constant.AudioGenerationTaskStatusConstant;
import cn.com.poc.audio_generation.entity.BizAudioGenerationTaskEntity;
import cn.com.poc.audio_generation.service.BizAudioGenerationTaskService;
import cn.com.poc.message.entity.AudioGenerationMessage;
import cn.com.poc.message.service.AudioGenerationConsumerService;
import cn.com.poc.message.topic.AudioGenerationTopic;
import cn.com.poc.thirdparty.resource.tencent.service.TencentService;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.com.yict.framemax.tumbleweed.client.annotation.Consumer;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
public class AudioGenerationConsumerServiceImpl implements AudioGenerationConsumerService {

    @Resource
    private BizAudioGenerationTaskService bizAudioGenerationTaskService;

    @Resource
    private TencentService tencentService;

    @Override
    @Consumer(topic = AudioGenerationTopic.AUDIO_GENERATION_TOPIC)
    public void generateAudio(AudioGenerationMessage message) {
        Long taskId = message.getTaskId();
        BizAudioGenerationTaskEntity taskEntity = bizAudioGenerationTaskService.get(taskId);
        if (taskEntity == null) {
            return;
        }
        try {
            String audioUrl = tencentService.textToAudio(taskEntity.getTextContent(), taskEntity.getVoiceId());
            //更新状态-[成功]
            taskEntity.setTaskStatus(AudioGenerationTaskStatusConstant.Success);
            taskEntity.setAudioUrl(audioUrl);
            bizAudioGenerationTaskService.update(taskEntity);
        } catch (Exception e) {
            //更新状态-[失败]
            taskEntity.setTaskStatus(AudioGenerationTaskStatusConstant.Fail);
            taskEntity.setTaskInfo(e.getMessage());
            bizAudioGenerationTaskService.update(taskEntity);
            throw new BusinessException("生成音频失败");
        }
    }
}
