package cn.com.poc.spokenscene.aggregation.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.aggregation.SpokenSceneCategorizationService;
import cn.com.poc.spokenscene.entity.BizSpokenSceneCategorizationEntity;
import cn.com.poc.spokenscene.service.BizSpokenSceneCategorizationService;
import cn.com.yict.framemax.core.exception.BusinessException;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

@Component
public class SpokenSceneCategorizationServiceImpl implements SpokenSceneCategorizationService {

    final private Logger logger = LoggerFactory.getLogger(SpokenSceneCategorizationService.class);

    @Resource
    private BizSpokenSceneCategorizationService bizSpokenSceneCategorizationService;

    @Override
    @Transactional
    public BizSpokenSceneCategorizationEntity createSceneCategorization(BizSpokenSceneCategorizationEntity bizSpokenSceneCategorizationEntity) {
        // 1. 判断是否已经存在该分类
        BizSpokenSceneCategorizationEntity findExample = new BizSpokenSceneCategorizationEntity();
        findExample.setSceneCategorizationTitle(bizSpokenSceneCategorizationEntity.getSceneCategorizationTitle());
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizSpokenSceneCategorizationEntity> entities = bizSpokenSceneCategorizationService.findByExample(findExample, null);
        if (CollectionUtils.isNotEmpty(entities)) {
            throw new BusinessException("该分类已存在，请勿重复创建");
        }
        //2. 重新排序
        findExample = new BizSpokenSceneCategorizationEntity();
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        entities = bizSpokenSceneCategorizationService.findByExample(findExample, null);
        for (BizSpokenSceneCategorizationEntity entity : entities) {
            entity.setSorted(entity.getSorted() + 1);
            BizSpokenSceneCategorizationEntity update = bizSpokenSceneCategorizationService.update(entity);
            if (null == update) {
                throw new BusinessException("创建失败");
            }
        }
        //3. 创建分类
        bizSpokenSceneCategorizationEntity.setSorted(0);
        return bizSpokenSceneCategorizationService.save(bizSpokenSceneCategorizationEntity);
    }

    @Override
    public Collection<BizSpokenSceneCategorizationEntity> getSceneCategorizations() {
        BizSpokenSceneCategorizationEntity findExample = new BizSpokenSceneCategorizationEntity();
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        return bizSpokenSceneCategorizationService.findByExample(findExample, null);
    }

    @Override
    @Transactional
    public BizSpokenSceneCategorizationEntity sortedSceneCategorization(Long sceneCategorizationId, Integer location) {
        BizSpokenSceneCategorizationEntity bizSpokenSceneCategorizationEntity = bizSpokenSceneCategorizationService.get(sceneCategorizationId);
        if (null == bizSpokenSceneCategorizationEntity) {
            throw new BusinessException("分类不存在");
        }
        Integer originalSorted = bizSpokenSceneCategorizationEntity.getSorted();
        if (Objects.equals(location, originalSorted)) {
            logger.info("原位置没改变");
            return bizSpokenSceneCategorizationEntity;
        }
        BizSpokenSceneCategorizationEntity findExample = new BizSpokenSceneCategorizationEntity();
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizSpokenSceneCategorizationEntity> entities = bizSpokenSceneCategorizationService.findByExample(findExample, null);
        //改变排序
        for (BizSpokenSceneCategorizationEntity entity : entities) {
            if (entity.getSceneCategorizationId().equals(sceneCategorizationId)) {
                continue;
            }
            if (location < originalSorted) {// 目标前移
                if (entity.getSorted() >= location && entity.getSorted() <= originalSorted) {
                    entity.setSorted(entity.getSorted() + 1);
                }
            } else {
                if (entity.getSorted() <= location && entity.getSorted() >= originalSorted) {
                    entity.setSorted(entity.getSorted() - 1);
                }
            }
            bizSpokenSceneCategorizationService.update(entity);
        }
        //更新目标
        bizSpokenSceneCategorizationEntity.setSorted(location);
        return bizSpokenSceneCategorizationService.update(bizSpokenSceneCategorizationEntity);
    }

    @Override
    public BizSpokenSceneCategorizationEntity updateSceneCategorization(BizSpokenSceneCategorizationEntity
                                                                                bizSpokenSceneCategorizationEntity) {
        Assert.notNull(bizSpokenSceneCategorizationEntity, "分类不能为空");
        Assert.notNull(bizSpokenSceneCategorizationEntity.getSceneCategorizationId(), "分类ID不能为空");
        //不能使用该方法改变排序 和 删除状态
        bizSpokenSceneCategorizationEntity.setSorted(null);
        bizSpokenSceneCategorizationEntity.setIsDeleted(null);
        return bizSpokenSceneCategorizationService.update(bizSpokenSceneCategorizationEntity);
    }

    @Override
    @Transactional
    public void delSceneCategorizations(Long sceneCategorizationId) {
        Assert.notNull(sceneCategorizationId, "分类ID不能为空");
        bizSpokenSceneCategorizationService.deleteById(sceneCategorizationId);
        BizSpokenSceneCategorizationEntity findExample = new BizSpokenSceneCategorizationEntity();
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizSpokenSceneCategorizationEntity> entities = bizSpokenSceneCategorizationService.findByExample(findExample, null);
        int order = 0;
        for (BizSpokenSceneCategorizationEntity entity : entities) {
            entity.setSorted(order++);
            bizSpokenSceneCategorizationService.update(entity);
        }
    }
}
