package cn.com.poc.spokenscene.aggregation.impl;

import cn.com.poc.audio_generation.aggregation.AudioGenerationService;
import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.spokenscene.aggregation.SpokenSceneConversationService;
import cn.com.poc.spokenscene.entity.*;
import cn.com.poc.spokenscene.query.SpokenSceneConversationQueryCondition;
import cn.com.poc.spokenscene.query.SpokenSceneConversationQueryItem;
import cn.com.poc.spokenscene.service.BizSpokenSceneConversationService;
import cn.com.poc.spokenscene.service.BizSpokenSentencesAudioGenerationRelationService;
import cn.com.poc.spokenscene.service.BizSpokenSentencesService;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.hutool.core.lang.Assert;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Component
public class SpokenSceneConversationServiceImpl implements SpokenSceneConversationService {

    @Resource
    private BizSpokenSceneConversationService bizSpokenSceneConversationService;

    @Resource
    private BizSpokenSentencesAudioGenerationRelationService bizSpokenSentencesAudioGenerationRelationService;

    @Resource
    private BizSpokenSentencesService bizSpokenSentencesService;

    @Resource
    private AudioGenerationService audioGenerationService;

    @Override
    public Object generatedConversation(String sceneContent) {
        return null;
    }

    @Override
    @Transactional
    public void saveConversation(SaveSpokenSceneConversationEntity entity) {
        Assert.notNull(entity, "参数不能为空");
        Assert.notNull(entity.getSpokenSceneId(), "场景ID不能为空");
        Assert.notEmpty(entity.getConversation(), "对话情景配置不能为空");

        Long spokenSceneId = entity.getSpokenSceneId();
        //删除原有的对话配置
        BizSpokenSceneConversationEntity findExample = new BizSpokenSceneConversationEntity();
        findExample.setSpokenSceneId(spokenSceneId);
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizSpokenSceneConversationEntity> entitys = bizSpokenSceneConversationService.findByExample(findExample, null);
        if (CollectionUtils.isNotEmpty(entitys)) {
            for (BizSpokenSceneConversationEntity bizSpokenSceneConversationEntity : entitys) {
                bizSpokenSceneConversationService.deleteById(bizSpokenSceneConversationEntity.getConversationId());
            }
        }

        List<SaveSpokenSceneConversationEntity.Conversation> conversations = entity.getConversation();
        int sorted = 0;
        for (SaveSpokenSceneConversationEntity.Conversation conversation : conversations) {
            BizSpokenSceneConversationEntity bizSpokenSceneConversationEntity = new BizSpokenSceneConversationEntity();
            bizSpokenSceneConversationEntity.setSentencesId(conversation.getSentencesId());
            bizSpokenSceneConversationEntity.setRoleName(conversation.getRoleName());
            bizSpokenSceneConversationEntity.setSorted(sorted++);
            bizSpokenSceneConversationEntity.setSpokenSceneId(spokenSceneId);
            bizSpokenSceneConversationEntity.setSentencesId(conversation.getSentencesId());
            bizSpokenSceneConversationEntity.setIsDeleted(CommonConstant.IsDeleted.N);
            bizSpokenSceneConversationService.save(bizSpokenSceneConversationEntity);
        }
    }

    @Override
    public List<SpokenSceneConversationQueryItem> getConversation(Long spokenSceneId) {
        Assert.notNull(spokenSceneId, "场景ID不能为空");
        SpokenSceneConversationQueryCondition spokenSceneConversationQueryCondition = new SpokenSceneConversationQueryCondition();
        spokenSceneConversationQueryCondition.setSpokenSceneId(spokenSceneId);
        return bizSpokenSceneConversationService.spokenSceneConversationQuery(spokenSceneConversationQueryCondition, null);
    }

    @Override
    @Transactional
    public List<BizSpokenSentencesAudioGenerationRelationEntity> generatedSentencesAudioTask(List<GeneratedSentencesAudioEntity> generatedSentencesAudioEntities) {
        Assert.notEmpty(generatedSentencesAudioEntities, "参数不能为空");
        List<BizSpokenSentencesAudioGenerationRelationEntity> result = new ArrayList<>();
        for (GeneratedSentencesAudioEntity generatedSentencesAudioEntity : generatedSentencesAudioEntities) {
            String cantoneseContent = generatedSentencesAudioEntity.getCantoneseContent();
            String mandarinContent = generatedSentencesAudioEntity.getMandarinContent();
            String cantoneseTranscription = generatedSentencesAudioEntity.getCantoneseTranscription();
            String voiceId = generatedSentencesAudioEntity.getVoiceId();
            if (StringUtils.isBlank(cantoneseContent)) {
                throw new IllegalArgumentException("粤语句子不能为空");
            }

            //保存口语句子
            BizSpokenSentencesEntity bizSpokenSentencesEntity = new BizSpokenSentencesEntity();
            bizSpokenSentencesEntity.setCantoneseContent(cantoneseContent);
            bizSpokenSentencesEntity.setCantoneseTranscription(cantoneseTranscription);
            bizSpokenSentencesEntity.setMandarinContent(mandarinContent);
            BizSpokenSentencesEntity spokenSentencesEntity = bizSpokenSentencesService.save(bizSpokenSentencesEntity);

            //创建文本转语音任务
            Long taskId = audioGenerationService.createTextToAudioTask(cantoneseContent, voiceId, 1.0);
            //保存关联
            BizSpokenSentencesAudioGenerationRelationEntity bizSpokenSentencesAudioGenerationRelationEntity = new BizSpokenSentencesAudioGenerationRelationEntity();
            bizSpokenSentencesAudioGenerationRelationEntity.setSentencesId(spokenSentencesEntity.getSentencesId());
            bizSpokenSentencesAudioGenerationRelationEntity.setTaskId(taskId);
            BizSpokenSentencesAudioGenerationRelationEntity relationEntity = bizSpokenSentencesAudioGenerationRelationService.save(bizSpokenSentencesAudioGenerationRelationEntity);

            result.add(relationEntity);
        }
        return result;
    }
}
