package cn.com.poc.spokenscene.aggregation.impl;

import cn.com.poc.audio_generation.constant.AudioGenerationTaskStatusConstant;
import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.spokenscene.aggregation.SpokenSceneService;
import cn.com.poc.spokenscene.constant.SpokenSceneConstant;
import cn.com.poc.spokenscene.entity.BizSpokenSceneCategorizationEntity;
import cn.com.poc.spokenscene.entity.BizSpokenSceneEntity;
import cn.com.poc.spokenscene.query.SpokenSceneConversationQueryCondition;
import cn.com.poc.spokenscene.query.SpokenSceneConversationQueryItem;
import cn.com.poc.spokenscene.query.SpokenSceneWordsQueryCondition;
import cn.com.poc.spokenscene.query.SpokenSceneWordsQueryItem;
import cn.com.poc.spokenscene.service.*;
import cn.com.yict.framemax.core.exception.BusinessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

@Component
public class SpokenSceneServiceImpl implements SpokenSceneService {

    final private Logger logger = LoggerFactory.getLogger(SpokenSceneServiceImpl.class);

    @Resource
    private BizSpokenSceneService bizSpokenSceneService;

    @Resource
    private BizSpokenSceneCategorizationService bizSpokenSceneCategorizationService;

    @Resource
    private BizSpokenSceneWordsService bizSpokenSceneWordsService;

    @Resource
    private BizSpokenSceneConversationService bizSpokenSceneConversationService;


    @Override
    @Transactional
    public BizSpokenSceneEntity createSpokenScene(BizSpokenSceneEntity bizSpokenSceneEntity) {
        Long sceneCategorizationId = bizSpokenSceneEntity.getSceneCategorizationId();
        BizSpokenSceneCategorizationEntity categorizationEntity = bizSpokenSceneCategorizationService.get(sceneCategorizationId);
        if (null == categorizationEntity) {
            throw new BusinessException("场景分类不存在");
        }
        //重名判断
        if (bizSpokenSceneService.checkRenameToTitle(sceneCategorizationId, bizSpokenSceneEntity.getSpokenSceneTitle(), null)) {
            throw new BusinessException("场景名称已存在");
        }

        //重新排序
        BizSpokenSceneEntity findExample = new BizSpokenSceneEntity();
        findExample.setSceneCategorizationId(sceneCategorizationId);
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizSpokenSceneEntity> entities = bizSpokenSceneService.findByExample(findExample, null);
        for (BizSpokenSceneEntity entity : entities) {
            entity.setSorted(entity.getSorted() + 1);
            bizSpokenSceneService.update(entity);
        }

        //初始化部分参数
        bizSpokenSceneEntity.setPostingStatus(SpokenSceneConstant.POSTING_STATUS.UN_PUBLISHED);
        bizSpokenSceneEntity.setSorted(0);
        return bizSpokenSceneService.save(bizSpokenSceneEntity);
    }

    @Override
    public BizSpokenSceneEntity updateSpokenScene(BizSpokenSceneEntity bizSpokenSceneEntity) {
        //重名判断
        if (bizSpokenSceneService.checkRenameToTitle(bizSpokenSceneEntity.getSpokenSceneId(), bizSpokenSceneEntity.getSpokenSceneTitle(), bizSpokenSceneEntity.getSpokenSceneId())) {
            throw new BusinessException("场景名称已存在");
        }

        //初始化部分参数 [不允许使用该方法更新 发布状态、排序、所在分类、是否删除 ]
        bizSpokenSceneEntity.setPostingStatus(null);
        bizSpokenSceneEntity.setSorted(null);
        bizSpokenSceneEntity.setIsDeleted(null);
        bizSpokenSceneEntity.setSceneCategorizationId(null);

        return bizSpokenSceneService.update(bizSpokenSceneEntity);
    }

    @Override
    @Transactional
    public void delSpokenScenes(Long spokenSceneId) {
        BizSpokenSceneEntity bizSpokenSceneEntity = bizSpokenSceneService.get(spokenSceneId);
        if (bizSpokenSceneEntity == null) {
            throw new BusinessException("场景不存在");
        }
        bizSpokenSceneService.deleteById(spokenSceneId);
        // 删除后，重新排序
        BizSpokenSceneEntity findExample = new BizSpokenSceneEntity();
        findExample.setSceneCategorizationId(bizSpokenSceneEntity.getSceneCategorizationId());
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizSpokenSceneEntity> entities = bizSpokenSceneService.findByExample(findExample, null);
        int sorted = 0;
        for (BizSpokenSceneEntity entity : entities) {
            entity.setSorted(sorted++);
            bizSpokenSceneService.update(entity);
        }
    }

    @Override
    public Collection<BizSpokenSceneEntity> getSpokenSceneCollection(Long sceneCategorizationId) {
        BizSpokenSceneCategorizationEntity categorizationEntity = bizSpokenSceneCategorizationService.get(sceneCategorizationId);
        if (null == categorizationEntity) {
            throw new BusinessException("无法获取该场景分类下的场景，因该场景分类已删除");
        }

        BizSpokenSceneEntity bizSpokenSceneEntity = new BizSpokenSceneEntity();
        bizSpokenSceneEntity.setSceneCategorizationId(sceneCategorizationId);
        bizSpokenSceneEntity.setIsDeleted(CommonConstant.IsDeleted.N);
        return bizSpokenSceneService.findByExample(bizSpokenSceneEntity, null);
    }

    @Override
    public BizSpokenSceneEntity getSpokenScene(Long spokenSceneId) {
        BizSpokenSceneEntity bizSpokenSceneEntity = bizSpokenSceneService.get(spokenSceneId);
        if (null == bizSpokenSceneEntity) {
            throw new BusinessException("场景不存在");
        }
        return bizSpokenSceneEntity;
    }

    @Override
    @Transactional
    public BizSpokenSceneEntity sortedSpokenScene(Long spokenSceneId, Integer location) {
        BizSpokenSceneEntity bizSpokenSceneEntity = bizSpokenSceneService.get(spokenSceneId);
        if (null == bizSpokenSceneEntity) {
            throw new BusinessException("场景不存在");
        }
        Integer originalSorted = bizSpokenSceneEntity.getSorted();
        if (Objects.equals(location, originalSorted)) {
            logger.info("------------- 原位置没改变 -------------");
            return bizSpokenSceneEntity;
        }
        BizSpokenSceneEntity findExample = new BizSpokenSceneEntity();
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizSpokenSceneEntity> entities = bizSpokenSceneService.findByExample(findExample, null);
        //改变排序
        for (BizSpokenSceneEntity entity : entities) {
            if (entity.getSceneCategorizationId().equals(spokenSceneId)) {
                continue;
            }
            if (location < originalSorted) {// 目标前移
                if (entity.getSorted() >= location && entity.getSorted() <= originalSorted) {
                    entity.setSorted(entity.getSorted() + 1);
                }
            } else {
                if (entity.getSorted() <= location && entity.getSorted() >= originalSorted) {
                    entity.setSorted(entity.getSorted() - 1);
                }
            }
            bizSpokenSceneService.update(entity);
        }
        //更新目标
        bizSpokenSceneEntity.setSorted(location);
        return bizSpokenSceneService.update(bizSpokenSceneEntity);
    }

    @Override
    public BizSpokenSceneEntity postingSpokenScene(Long spokenSceneId) {
        BizSpokenSceneEntity bizSpokenSceneEntity = bizSpokenSceneService.get(spokenSceneId);
        if (null == bizSpokenSceneEntity) {
            throw new BusinessException("场景不存在");
        }
        String postingStatus = bizSpokenSceneEntity.getPostingStatus();
        if (SpokenSceneConstant.POSTING_STATUS.PUBLISHED.equals(postingStatus)) {
            bizSpokenSceneEntity.setPostingStatus(SpokenSceneConstant.POSTING_STATUS.OFF_SHELF);
        } else {
            //发布校验
            publishingCalibration(spokenSceneId);
            bizSpokenSceneEntity.setPostingStatus(SpokenSceneConstant.POSTING_STATUS.PUBLISHED);
        }
        return bizSpokenSceneService.update(bizSpokenSceneEntity);
    }

    /**
     * 发布校验
     *
     * @param spokenSceneId
     */
    private void publishingCalibration(Long spokenSceneId) {
        // 1. 词汇音频训练校验
        SpokenSceneWordsQueryCondition spokenSceneWordsQueryCondition = new SpokenSceneWordsQueryCondition();
        spokenSceneWordsQueryCondition.setSpokenSceneId(spokenSceneId);
        List<SpokenSceneWordsQueryItem> spokenSceneWordsQueryItems = bizSpokenSceneWordsService.spokenSceneWordsQuery(spokenSceneWordsQueryCondition, null);
        for (SpokenSceneWordsQueryItem item : spokenSceneWordsQueryItems) {
            if (!AudioGenerationTaskStatusConstant.Success.equals(item.getTaskStatus())) {
                throw new BusinessException("该场景下词汇音频生成未完成或训练失败，无法发布");
            }
        }

        // 2. 情景对话音频校验
        SpokenSceneConversationQueryCondition spokenSceneConversationQueryCondition = new SpokenSceneConversationQueryCondition();
        spokenSceneConversationQueryCondition.setSpokenSceneId(spokenSceneId);
        List<SpokenSceneConversationQueryItem> spokenSceneConversationQueryItems = bizSpokenSceneConversationService.spokenSceneConversationQuery(spokenSceneConversationQueryCondition, null);
        for (SpokenSceneConversationQueryItem item : spokenSceneConversationQueryItems) {
            if (!AudioGenerationTaskStatusConstant.Success.equals(item.getTaskStatus())) {
                throw new BusinessException("该场景下情景对话音频生成未完成或失败，无法发布");
            }
        }
    }
}
