package cn.com.poc.spokenscene.aggregation.impl;

import cn.com.poc.audio_generation.aggregation.AudioGenerationService;
import cn.com.poc.audio_generation.constant.AudioGenerationTaskStatusConstant;
import cn.com.poc.audio_generation.entity.BizAudioGenerationTaskEntity;
import cn.com.poc.audio_generation.service.BizAudioGenerationTaskService;
import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.aggregation.SpokenSceneWordsService;
import cn.com.poc.spokenscene.entity.BizSpokenSceneWordsEntity;
import cn.com.poc.spokenscene.entity.BizSpokenWordsAudioGenerationRelationEntity;
import cn.com.poc.spokenscene.query.SpokenSceneWordsQueryCondition;
import cn.com.poc.spokenscene.query.SpokenSceneWordsQueryItem;
import cn.com.poc.spokenscene.service.BizSpokenSceneService;
import cn.com.poc.spokenscene.service.BizSpokenSceneWordsService;
import cn.com.poc.spokenscene.service.BizSpokenWordsAudioGenerationRelationService;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class SpokenSceneWordsServiceImpl implements SpokenSceneWordsService {

    final Logger logger = LoggerFactory.getLogger(SpokenSceneWordsService.class);

    @Resource
    private BizSpokenWordsAudioGenerationRelationService bizSpokenWordsAudioGenerationRelationService;

    @Resource
    private BizSpokenSceneWordsService bizSpokenSceneWordsService;

    @Resource
    private AudioGenerationService audioGenerationService;

    @Override
    @Transactional
    public BizSpokenSceneWordsEntity createSpokenSceneWords(BizSpokenSceneWordsEntity bizSpokenSceneWordsEntity) {
        Assert.notNull(bizSpokenSceneWordsEntity, "bizSpokenSceneWordsEntity is null");
        Assert.notNull(bizSpokenSceneWordsEntity.getCantoneseWord(), "cantonese is null");
        Assert.notNull(bizSpokenSceneWordsEntity.getMandarinWord(), "mandarin word is null");

        //保存单词
        BizSpokenSceneWordsEntity spokenSceneWordsEntity = bizSpokenSceneWordsService.save(bizSpokenSceneWordsEntity);

        //音频生成任务
        Long taskId = audioGenerationService.createTextToAudioTask(spokenSceneWordsEntity.getCantoneseWord(), "1", 0.5);

        //保存关联
        BizSpokenWordsAudioGenerationRelationEntity bizSpokenWordsAudioGenerationRelationEntity = new BizSpokenWordsAudioGenerationRelationEntity();
        bizSpokenWordsAudioGenerationRelationEntity.setWordId(spokenSceneWordsEntity.getWordId());
        bizSpokenWordsAudioGenerationRelationEntity.setTaskId(taskId);
        bizSpokenWordsAudioGenerationRelationService.save(bizSpokenWordsAudioGenerationRelationEntity);

        return spokenSceneWordsEntity;
    }

    @Override
    @Transactional
    public BizSpokenSceneWordsEntity updateSpokenSceneWords(BizSpokenSceneWordsEntity bizSpokenSceneWordsEntity) {
        Assert.notNull(bizSpokenSceneWordsEntity, "bizSpokenSceneWordsEntity is null");
        Assert.notNull(bizSpokenSceneWordsEntity.getCantoneseWord(), "cantonese is null");
        Assert.notNull(bizSpokenSceneWordsEntity.getMandarinWord(), "mandarin word is null");

        //保存单词
        BizSpokenSceneWordsEntity spokenSceneWordsEntity = bizSpokenSceneWordsService.update(bizSpokenSceneWordsEntity);
        BizSpokenWordsAudioGenerationRelationEntity relationEntity = bizSpokenWordsAudioGenerationRelationService.getByWordId(spokenSceneWordsEntity.getWordId());
        if (relationEntity != null) {
            bizSpokenWordsAudioGenerationRelationService.del(relationEntity.getRelationId());
        }

        //音频生成任务
        Long taskId = audioGenerationService.createTextToAudioTask(spokenSceneWordsEntity.getCantoneseWord(), "1", 0.5);

        //保存关联
        BizSpokenWordsAudioGenerationRelationEntity bizSpokenWordsAudioGenerationRelationEntity = new BizSpokenWordsAudioGenerationRelationEntity();
        bizSpokenWordsAudioGenerationRelationEntity.setWordId(spokenSceneWordsEntity.getWordId());
        bizSpokenWordsAudioGenerationRelationEntity.setTaskId(taskId);
        bizSpokenWordsAudioGenerationRelationService.save(bizSpokenWordsAudioGenerationRelationEntity);

        return spokenSceneWordsEntity;
    }

    @Override
    public void delSpokenSceneWords(Long wordId) {
        bizSpokenSceneWordsService.deleteById(wordId);
    }

    @Override
    @Transactional
    public BizSpokenSceneWordsEntity sortedSpokenSceneWords(Long wordId, Integer location) {
        Assert.notNull(wordId, "wordId is null");
        Assert.notNull(location, "location is null");
        BizSpokenSceneWordsEntity bizSpokenSceneWordsEntity = bizSpokenSceneWordsService.get(wordId);
        if (bizSpokenSceneWordsEntity == null) {
            logger.error("wordId is not exist");
        }
        Integer originalSorted = bizSpokenSceneWordsEntity.getSorted();

        BizSpokenSceneWordsEntity findExample = new BizSpokenSceneWordsEntity();
        findExample.setSpokenSceneId(bizSpokenSceneWordsEntity.getSpokenSceneId());
        findExample.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizSpokenSceneWordsEntity> wordsEntities = bizSpokenSceneWordsService.findByExample(findExample, null);

        for (BizSpokenSceneWordsEntity entity : wordsEntities) {
            if (entity.getWordId().equals(wordId)) {
                continue;
            }
            if (location < originalSorted) {// 目标前移
                if (entity.getSorted() >= location && entity.getSorted() <= originalSorted) {
                    entity.setSorted(entity.getSorted() + 1);
                }
            } else {
                if (entity.getSorted() <= location && entity.getSorted() >= originalSorted) {
                    entity.setSorted(entity.getSorted() - 1);
                }
            }
            bizSpokenSceneWordsService.update(entity);
        }
        bizSpokenSceneWordsEntity.setSorted(location);
        return bizSpokenSceneWordsService.update(bizSpokenSceneWordsEntity);
    }

    @Override
    public Collection<SpokenSceneWordsQueryItem> getSpokenSceneWordsCollection(Long spokenSceneId) {
        SpokenSceneWordsQueryCondition spokenSceneWordsQueryCondition = new SpokenSceneWordsQueryCondition();
        spokenSceneWordsQueryCondition.setSpokenSceneId(spokenSceneId);
        return bizSpokenSceneWordsService.spokenSceneWordsQuery(spokenSceneWordsQueryCondition, null);
    }
}
