package cn.com.poc.spokenscene.rest;

import cn.com.poc.spokenscene.dto.SceneCategorizationDto;
import cn.com.poc.spokenscene.dto.SpokenSceneDto;
import cn.com.yict.framemax.core.rest.BaseRest;
import cn.com.yict.framemax.web.permission.Access;
import cn.com.yict.framemax.web.permission.Permission;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Collection;

@Permission(Access.Safety)
public interface SpokenSceneRest extends BaseRest {

    /**
     * 创建场景分类
     *
     * @param sceneCategorizationDto 场景分类
     */
    SceneCategorizationDto createSceneCategorization(@RequestBody SceneCategorizationDto sceneCategorizationDto) throws Exception;

    /**
     * 场景分类列表
     */
    Collection<SceneCategorizationDto> getSceneCategorizations();

    /**
     * 场景分类排序
     *
     * @param sceneCategorizationId 场景分类id
     * @param location              排序位置
     */
    SceneCategorizationDto sortedSceneCategorization(@RequestParam Long sceneCategorizationId, @RequestParam Integer location);

    /**
     * 更新场景分类详情
     *
     * @param sceneCategorizationDto 场景分类
     */
    SceneCategorizationDto updateSceneCategorization(@RequestBody SceneCategorizationDto sceneCategorizationDto);

    /**
     * 删除场景分类
     *
     * @param sceneCategorizationId 场景分类id
     */
    void delSceneCategorizations(@RequestParam Long sceneCategorizationId);


    /**
     * 创建场景
     *
     * @param spokenSceneDto 口语场景
     */
    SpokenSceneDto createSpokenScene(@RequestBody SpokenSceneDto spokenSceneDto);

    /**
     * 更新场景
     *
     * @param spokenSceneDto 口语场景
     */
    SpokenSceneDto updateSpokenScene(@RequestBody SpokenSceneDto spokenSceneDto);

    /**
     * 删除场景
     *
     * @param spokenSceneId 口语场景id
     */
    void delSpokenScenes(@RequestParam Long spokenSceneId);

    /**
     * 场景列表
     *
     * @param sceneCategorizationId 场景分类id
     */
    Collection<SpokenSceneDto> getSpokenSceneCollection(@RequestParam Long sceneCategorizationId);

    /**
     * 口语场景详情
     *
     * @param spokenSceneId 口语场景id
     */
    SpokenSceneDto getSpokenScene(@RequestParam Long spokenSceneId);

    /**
     * 口语场景排序
     *
     * @param spokenSceneId 口语场景id
     * @param location      排序位置
     * @return
     */
    SpokenSceneDto sortedSpokenScene(@RequestParam Long spokenSceneId, @RequestParam Integer location);


    /**
     * 发布\下架 口语场景
     *
     * @param spokenSceneId 口语场景id
     */
    SpokenSceneDto postingSpokenScene(@RequestParam Long spokenSceneId);
}
