package cn.com.poc.spokenscene.rest.impl;

import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.aggregation.SpokenSceneConversationService;
import cn.com.poc.spokenscene.dto.*;
import cn.com.poc.spokenscene.entity.BizSpokenSentencesAudioGenerationRelationEntity;
import cn.com.poc.spokenscene.entity.GeneratedSentencesAudioEntity;
import cn.com.poc.spokenscene.entity.SaveSpokenSceneConversationEntity;
import cn.com.poc.spokenscene.query.SpokenSceneConversationQueryItem;
import cn.com.poc.spokenscene.rest.SpokenSceneConversationRest;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


@Component
public class SpokenSceneConversationRestImpl implements SpokenSceneConversationRest {

    @Resource
    private SpokenSceneConversationService spokenSceneConversationService;

    @Override
    public Object generatedConversation(GeneratedConversationDto dto) {
        return null;
    }

    @Override
    public void saveConversation(SaveSpokenSceneConversationDto saveSpokenSceneConversationDto) {
        Assert.notNull(saveSpokenSceneConversationDto, "参数不能为空");
        Assert.notNull(saveSpokenSceneConversationDto.getSpokenSceneId(), "参数不能为空");
        Assert.notEmpty(saveSpokenSceneConversationDto.getConversation(), "参数不能为空");

        SaveSpokenSceneConversationEntity saveSpokenSceneConversationEntity = new SaveSpokenSceneConversationEntity();
        BeanUtils.copyProperties(saveSpokenSceneConversationDto, saveSpokenSceneConversationEntity);
        spokenSceneConversationService.saveConversation(saveSpokenSceneConversationEntity);
    }

    @Override
    public List<SpokenSceneConversationDto> getConversation(Long spokenSceneId) {
        Assert.notNull(spokenSceneId, "参数不能为空");
        List<SpokenSceneConversationDto> result = new ArrayList<>();
        List<SpokenSceneConversationQueryItem> conversationQueryItems = spokenSceneConversationService.getConversation(spokenSceneId);
        if (CollectionUtils.isNotEmpty(conversationQueryItems)) {
            result = conversationQueryItems.stream().map(item -> {
                SpokenSceneConversationDto dto = new SpokenSceneConversationDto();
                dto.setConversationId(item.getConversationId());
                dto.setCantoneseContent(item.getCantoneseContent());
                dto.setCantoneseTranscription(item.getCantoneseTranscription());
                dto.setMandarinContent(item.getMandarinContent());
                dto.setTaskStatus(item.getTaskStatus());
                dto.setAudioUrl(item.getAudioUrl());
                dto.setRoleName(item.getRoleName());
                dto.setSorted(item.getSorted());
                return dto;
            }).collect(Collectors.toList());
        }

        return result;
    }

    @Override
    public List<GeneratedSentencesAudioResultDto> generatedSentencesAudioTask(List<GeneratedSentencesAudioDto> generatedSentencesAudioDtoList) {
        Assert.notEmpty(generatedSentencesAudioDtoList, "参数不能为空");
        List<GeneratedSentencesAudioEntity> generatedSentencesAudioEntities = generatedSentencesAudioDtoList.stream().map(item -> {
            GeneratedSentencesAudioEntity entity = new GeneratedSentencesAudioEntity();
            BeanUtils.copyProperties(item, entity);
            return entity;
        }).collect(Collectors.toList());

        List<BizSpokenSentencesAudioGenerationRelationEntity> relationEntities = spokenSceneConversationService.generatedSentencesAudioTask(generatedSentencesAudioEntities);
        List<GeneratedSentencesAudioResultDto> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(relationEntities)) {
            result = relationEntities.stream().map(entity -> {
                GeneratedSentencesAudioResultDto dto = new GeneratedSentencesAudioResultDto();
                dto.setSentenceId(entity.getSentencesId());
                dto.setTaskId(entity.getTaskId());
                return dto;
            }).collect(Collectors.toList());
        }
        return result;
    }
}