package cn.com.poc.spokenscene.rest.impl;


import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.aggregation.SpokenSceneCategorizationService;
import cn.com.poc.spokenscene.aggregation.SpokenSceneService;
import cn.com.poc.spokenscene.constant.SpokenSceneConstant;
import cn.com.poc.spokenscene.convert.SpokenSceneCategorizationConvert;
import cn.com.poc.spokenscene.convert.SpokenSceneConvert;
import cn.com.poc.spokenscene.dto.SceneCategorizationDto;
import cn.com.poc.spokenscene.dto.SpokenSceneDto;
import cn.com.poc.spokenscene.entity.BizSpokenSceneCategorizationEntity;
import cn.com.poc.spokenscene.entity.BizSpokenSceneEntity;
import cn.com.poc.spokenscene.rest.SpokenSceneRest;
import cn.com.yict.framemax.core.exception.BusinessException;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.stream.Collectors;

@Component
public class SpokenSceneRestImpl implements SpokenSceneRest {

    @Resource
    private SpokenSceneCategorizationService spokenSceneCategorizationService;

    @Resource
    private SpokenSceneService spokenSceneService;

    @Override
    public SceneCategorizationDto createSceneCategorization(SceneCategorizationDto sceneCategorizationDto) throws BusinessException {
        Assert.notNull(sceneCategorizationDto, "sceneCategorizationDto is null");
        Assert.notBlank(sceneCategorizationDto.getSceneCategorizationTitle(), "sceneCategorizationDto.sceneName is null");
        BizSpokenSceneCategorizationEntity bizSpokenSceneCategorizationEntity = spokenSceneCategorizationService.createSceneCategorization(SpokenSceneCategorizationConvert.dtoToEntity(sceneCategorizationDto));
        if (bizSpokenSceneCategorizationEntity == null) {
            throw new BusinessException("保存场景分类失败");
        }
        return SpokenSceneCategorizationConvert.entityToDto(bizSpokenSceneCategorizationEntity);
    }

    @Override
    public Collection<SceneCategorizationDto> getSceneCategorizations() {
        Collection<SceneCategorizationDto> result = new ArrayList<>();
        Collection<BizSpokenSceneCategorizationEntity> categorizationEntities = spokenSceneCategorizationService.getSceneCategorizations();
        if (CollectionUtils.isNotEmpty(categorizationEntities)) {
            result = categorizationEntities.stream().map(SpokenSceneCategorizationConvert::entityToDto).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public SceneCategorizationDto sortedSceneCategorization(Long sceneCategorizationId, Integer location) {
        Assert.notNull(sceneCategorizationId, "sceneCategorizationId is null");
        Assert.notNull(location, "location is null");
        Assert.isTrue(location >= 0, "location must greater than or equal to 0");
        BizSpokenSceneCategorizationEntity bizSpokenSceneCategorizationEntity = spokenSceneCategorizationService.sortedSceneCategorization(sceneCategorizationId, location);
        return SpokenSceneCategorizationConvert.entityToDto(bizSpokenSceneCategorizationEntity);
    }

    @Override
    public SceneCategorizationDto updateSceneCategorization(SceneCategorizationDto sceneCategorizationDto) {
        Assert.notNull(sceneCategorizationDto, "sceneCategorizationDto is null");
        BizSpokenSceneCategorizationEntity bizSpokenSceneCategorizationEntity = spokenSceneCategorizationService.updateSceneCategorization(SpokenSceneCategorizationConvert.dtoToEntity(sceneCategorizationDto));
        return SpokenSceneCategorizationConvert.entityToDto(bizSpokenSceneCategorizationEntity);
    }

    @Override
    public void delSceneCategorizations(Long sceneCategorizationId) {
        Assert.notNull(sceneCategorizationId, "sceneCategorizationId is null");
        spokenSceneCategorizationService.delSceneCategorizations(sceneCategorizationId);
    }

    @Override
    public SpokenSceneDto createSpokenScene(SpokenSceneDto spokenSceneDto) {
        Assert.notNull(spokenSceneDto, "spokenSceneDto is null");
        Assert.notBlank(spokenSceneDto.getSpokenSceneTitle(), "spokenSceneTitle is null");
        Assert.isTrue(spokenSceneDto.getSpokenSceneTitle().length() <= 10, "title must less than or equal to 10");
        Assert.notNull(spokenSceneDto.getSceneCategorizationId(), "spokenSceneCategorizationId is null");
        Assert.notNull(spokenSceneDto.getDescriptive(), "descriptive is null");
        Assert.notNull(spokenSceneDto.getCoverUrl(), "cover is null");
        Assert.notNull(spokenSceneDto.getDifficultyLevel(), "difficulty level is null");
        Assert.isTrue(SpokenSceneConstant.DIFFICULTY_LEVEL.EASY.equals(spokenSceneDto.getDifficultyLevel())
                        || SpokenSceneConstant.DIFFICULTY_LEVEL.MIDDLE.equals(spokenSceneDto.getDifficultyLevel())
                        || SpokenSceneConstant.DIFFICULTY_LEVEL.HARD.equals(spokenSceneDto.getDifficultyLevel())
                , "difficulty level must is Easy , Middle,Hard");

        BizSpokenSceneEntity spokenSceneEntity = spokenSceneService.createSpokenScene(SpokenSceneConvert.dtoToEntity(spokenSceneDto));
        if (spokenSceneEntity == null) {
            throw new BusinessException("保存场景失败");
        }
        return SpokenSceneConvert.entityToDto(spokenSceneEntity);
    }

    @Override
    public SpokenSceneDto updateSpokenScene(SpokenSceneDto spokenSceneDto) {
        Assert.notNull(spokenSceneDto, "spokenSceneDto is null");
        Assert.notNull(spokenSceneDto.getSpokenSceneId(), "spokenSceneId is null");
        Assert.isTrue(spokenSceneDto.getSorted() != null, "this interface cannot change the stored");
        Assert.isTrue(spokenSceneDto.getPostingStatus() != null, "this interface cannot change the postingStatus");

        BizSpokenSceneEntity bizSpokenSceneEntity = spokenSceneService.updateSpokenScene(SpokenSceneConvert.dtoToEntity(spokenSceneDto));
        if (bizSpokenSceneEntity == null) {
            throw new BusinessException("更新场景失败");
        }

        return SpokenSceneConvert.entityToDto(bizSpokenSceneEntity);
    }

    @Override
    public void delSpokenScenes(Long spokenSceneId) {
        Assert.notNull(spokenSceneId, "spokenSceneId is null");
        spokenSceneService.delSpokenScenes(spokenSceneId);
    }

    @Override
    public Collection<SpokenSceneDto> getSpokenSceneCollection(Long sceneCategorizationId) {
        Assert.notNull(sceneCategorizationId, "sceneCategorizationId is null");
        Collection<BizSpokenSceneEntity> spokenSceneEntities = spokenSceneService.getSpokenSceneCollection(sceneCategorizationId);
        Collection<SpokenSceneDto> result = spokenSceneEntities.stream().map(SpokenSceneConvert::entityToDto).collect(Collectors.toList());
        return result;
    }

    @Override
    public SpokenSceneDto getSpokenScene(Long spokenSceneId) {
        Assert.notNull(spokenSceneId, "spokenSceneId is null");
        BizSpokenSceneEntity spokenSceneEntity = spokenSceneService.getSpokenScene(spokenSceneId);
        if (spokenSceneEntity == null) {
            throw new BusinessException("场景不存在");
        }
        return SpokenSceneConvert.entityToDto(spokenSceneEntity);
    }

    @Override
    public SpokenSceneDto sortedSpokenScene(Long spokenSceneId, Integer location) {
        Assert.notNull(spokenSceneId, "spokenSceneId is null");
        Assert.notNull(location, "location is null");
        Assert.isTrue(location >= 0, "location must greater than or equal to 0");
        BizSpokenSceneEntity bizSpokenSceneEntity = spokenSceneService.sortedSpokenScene(spokenSceneId, location);
        if (bizSpokenSceneEntity == null) {
            throw new BusinessException("排序失败");
        }
        return SpokenSceneConvert.entityToDto(bizSpokenSceneEntity);
    }

    @Override
    public SpokenSceneDto postingSpokenScene(Long spokenSceneId) {
        Assert.notNull(spokenSceneId, "spokenSceneId is null");
        BizSpokenSceneEntity bizSpokenSceneEntity = spokenSceneService.postingSpokenScene(spokenSceneId);
        if (bizSpokenSceneEntity == null) {
            throw new BusinessException("发布状态更新失败");
        }
        return SpokenSceneConvert.entityToDto(bizSpokenSceneEntity);
    }
}
