package cn.com.poc.spokenscene.rest.impl;

import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.aggregation.SpokenSceneWordsService;
import cn.com.poc.spokenscene.convert.SpokenSceneWordsConvert;
import cn.com.poc.spokenscene.dto.SpokenSceneWordsDto;
import cn.com.poc.spokenscene.entity.BizSpokenSceneWordsEntity;
import cn.com.poc.spokenscene.query.SpokenSceneWordsQueryItem;
import cn.com.poc.spokenscene.rest.SpokenSceneWordsRest;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;

@Component
public class SpokenSceneWordsRestImpl implements SpokenSceneWordsRest {

    @Resource
    private SpokenSceneWordsService spokenSceneWordsService;

    @Override
    public SpokenSceneWordsDto createSpokenSceneWords(SpokenSceneWordsDto spokenSceneWordsDto) {
        Assert.notNull(spokenSceneWordsDto, "spokenSceneWordsDto is null");
        BizSpokenSceneWordsEntity bizSpokenSceneWordsEntity = SpokenSceneWordsConvert.dtoToEntity(spokenSceneWordsDto);
        BizSpokenSceneWordsEntity sceneWordsEntity = spokenSceneWordsService.createSpokenSceneWords(bizSpokenSceneWordsEntity);
        return SpokenSceneWordsConvert.entityToDto(sceneWordsEntity);
    }

    @Override
    public SpokenSceneWordsDto updateSpokenSceneWords(SpokenSceneWordsDto spokenSceneWordsDto) {
        Assert.notNull(spokenSceneWordsDto, "spokenSceneWordsDto is null");
        BizSpokenSceneWordsEntity bizSpokenSceneWordsEntity = SpokenSceneWordsConvert.dtoToEntity(spokenSceneWordsDto);
        BizSpokenSceneWordsEntity sceneWordsEntity = spokenSceneWordsService.updateSpokenSceneWords(bizSpokenSceneWordsEntity);
        return SpokenSceneWordsConvert.entityToDto(sceneWordsEntity);
    }

    @Override
    public void delSpokenSceneWords(Long wordId) {
        Assert.notNull(wordId, "wordId is null");
        spokenSceneWordsService.delSpokenSceneWords(wordId);
    }

    @Override
    public SpokenSceneWordsDto sortedSpokenSceneWords(Long wordId, Integer location) {
        Assert.notNull(wordId, "wordId is null");
        Assert.notNull(location, "location is null");
        Assert.isTrue(location >= 0, "location must be greater than zero");
        BizSpokenSceneWordsEntity bizSpokenSceneWordsEntity = spokenSceneWordsService.sortedSpokenSceneWords(wordId, location);
        return SpokenSceneWordsConvert.entityToDto(bizSpokenSceneWordsEntity);
    }

    @Override
    public Collection<SpokenSceneWordsDto> getSpokenSceneWordsCollection(Long spokenSceneID) {
        Assert.notNull(spokenSceneID, "spokenSceneID is null");
        Collection<SpokenSceneWordsDto> result = new ArrayList<>();
        Collection<SpokenSceneWordsQueryItem> wordsQueryItems = spokenSceneWordsService.getSpokenSceneWordsCollection(spokenSceneID);
        if (CollectionUtils.isNotEmpty(wordsQueryItems)){
            for (SpokenSceneWordsQueryItem wordsQueryItem : wordsQueryItems) {
                SpokenSceneWordsDto dto = new SpokenSceneWordsDto();
                dto.setSpokenSceneId(wordsQueryItem.getSpokenSceneId());
                dto.setSorted(wordsQueryItem.getSorted());
                dto.setCantoneseWord(wordsQueryItem.getCantoneseWord());
                dto.setMandarinWord(wordsQueryItem.getMandarinWord());
                dto.setCantonesePronunciationsUrl(wordsQueryItem.getAudioUrl());
                dto.setWordId(wordsQueryItem.getWordId());
                dto.setGenerationStatus(wordsQueryItem.getTaskStatus());
            }
        }
        return result;
    }
}
