package cn.com.poc.spokenscene.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.convert.SpokenSceneCategorizationConvert;
import cn.com.poc.spokenscene.model.BizSpokenSceneCategorizationModel;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.poc.spokenscene.service.BizSpokenSceneCategorizationService;
import cn.com.poc.spokenscene.entity.BizSpokenSceneCategorizationEntity;
import cn.com.poc.spokenscene.repository.BizSpokenSceneCategorizationRepository;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import javax.annotation.Resource;

@Service
public class BizSpokenSceneCategorizationServiceImpl extends BaseServiceImpl
        implements BizSpokenSceneCategorizationService {

    @Resource
    private BizSpokenSceneCategorizationRepository repository;


    @Override
    public BizSpokenSceneCategorizationEntity get(Long id) {
        Assert.notNull(id, "id is null");
        BizSpokenSceneCategorizationModel bizSpokenSceneCategorizationModel = this.repository.get(id);
        if (bizSpokenSceneCategorizationModel == null || CommonConstant.IsDeleted.Y.equals(bizSpokenSceneCategorizationModel.getIsDeleted())) {
            return null;
        }
        return SpokenSceneCategorizationConvert.modelToEntity(bizSpokenSceneCategorizationModel);
    }

    @Override
    public List<BizSpokenSceneCategorizationEntity> findByExample(BizSpokenSceneCategorizationEntity entity, PagingInfo pagingInfo) {
        List<BizSpokenSceneCategorizationEntity> result = new ArrayList<>();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizSpokenSceneCategorizationModel model = SpokenSceneCategorizationConvert.entityToModel(entity);
        String orderBy = "sorted asc";
        List<BizSpokenSceneCategorizationModel> models = this.repository.findByExample(model, orderBy, pagingInfo);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(SpokenSceneCategorizationConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public void deleteById(Long id) {
        Assert.notNull(id, "id is null");
        BizSpokenSceneCategorizationModel bizSpokenSceneCategorizationModel = this.repository.get(id);
        if (null == bizSpokenSceneCategorizationModel) {
            return;
        }
        bizSpokenSceneCategorizationModel.setIsDeleted(CommonConstant.IsDeleted.Y);
        repository.save(bizSpokenSceneCategorizationModel);
    }

    @Override
    public BizSpokenSceneCategorizationEntity save(BizSpokenSceneCategorizationEntity entity) {
        Assert.notNull(entity, "entity is null");
        BizSpokenSceneCategorizationModel bizSpokenSceneCategorizationModel = SpokenSceneCategorizationConvert.entityToModel(entity);
        BizSpokenSceneCategorizationModel saveModel = this.repository.save(bizSpokenSceneCategorizationModel);
        return SpokenSceneCategorizationConvert.modelToEntity(saveModel);
    }

    @Override
    public BizSpokenSceneCategorizationEntity update(BizSpokenSceneCategorizationEntity entity) {
        Assert.notNull(entity, "entity is null");
        Assert.notNull(entity.getSceneCategorizationId(), "sceneCategorizationId is null");
        BizSpokenSceneCategorizationModel bizSpokenSceneCategorizationModel = this.repository.get(entity.getSceneCategorizationId());
        if (bizSpokenSceneCategorizationModel == null || CommonConstant.IsDeleted.Y.equals(bizSpokenSceneCategorizationModel.getIsDeleted())) {
            return null;
        }
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(entity, bizSpokenSceneCategorizationModel, copyOptions);
        BizSpokenSceneCategorizationModel saveModel = this.repository.save(bizSpokenSceneCategorizationModel);
        return SpokenSceneCategorizationConvert.modelToEntity(saveModel);
    }
}