package cn.com.poc.spokenscene.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.convert.SpokenSceneConversationConvert;
import cn.com.poc.spokenscene.convert.SpokenSceneConvert;
import cn.com.poc.spokenscene.entity.BizSpokenSceneConversationEntity;
import cn.com.poc.spokenscene.entity.BizSpokenSceneEntity;
import cn.com.poc.spokenscene.model.BizSpokenSceneModel;
import cn.com.poc.spokenscene.query.SpokenSceneConversationQueryCondition;
import cn.com.poc.spokenscene.query.SpokenSceneConversationQueryItem;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.poc.spokenscene.service.BizSpokenSceneConversationService;
import cn.com.poc.spokenscene.model.BizSpokenSceneConversationModel;
import cn.com.poc.spokenscene.repository.BizSpokenSceneConversationRepository;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;

@Service
public class BizSpokenSceneConversationServiceImpl extends BaseServiceImpl
        implements BizSpokenSceneConversationService {

    @Resource
    private BizSpokenSceneConversationRepository repository;

    @Override
    public BizSpokenSceneConversationEntity get(Long id) {
        Assert.notNull(id, "id is null");
        BizSpokenSceneConversationModel bizSpokenSceneConversationModel = this.repository.get(id);
        if (bizSpokenSceneConversationModel == null || CommonConstant.IsDeleted.Y.equals(bizSpokenSceneConversationModel.getIsDeleted())) {
            return null;
        }
        return SpokenSceneConversationConvert.modelToEntity(bizSpokenSceneConversationModel);
    }

    @Override
    public List<BizSpokenSceneConversationEntity> findByExample(BizSpokenSceneConversationEntity entity, PagingInfo pagingInfo) {
        List<BizSpokenSceneConversationEntity> result = new ArrayList<>();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizSpokenSceneConversationModel model = SpokenSceneConversationConvert.entityToModel(entity);
        String orderBy = "sorted asc";
        List<BizSpokenSceneConversationModel> models = this.repository.findByExample(model, orderBy, pagingInfo);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(SpokenSceneConversationConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public void deleteById(Long id) {
        Assert.notNull(id, "id is null");
        BizSpokenSceneConversationModel bizSpokenSceneConversationModel = this.repository.get(id);
        if (bizSpokenSceneConversationModel == null || CommonConstant.IsDeleted.Y.equals(bizSpokenSceneConversationModel.getIsDeleted())) {
            return;
        }
        bizSpokenSceneConversationModel.setIsDeleted(CommonConstant.IsDeleted.Y);
        this.repository.save(bizSpokenSceneConversationModel);
    }

    @Override
    public BizSpokenSceneConversationEntity save(BizSpokenSceneConversationEntity entity) {
        Assert.notNull(entity, "entity is null");
        entity.setConversationId(null);
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizSpokenSceneConversationModel saveModel = this.repository.save(SpokenSceneConversationConvert.entityToModel(entity));
        return SpokenSceneConversationConvert.modelToEntity(saveModel);
    }

    @Override
    public BizSpokenSceneConversationEntity update(BizSpokenSceneConversationEntity entity) {
        Assert.notNull(entity, "entity is null");
        Assert.notNull(entity.getSpokenSceneId(), "spokenSceneId is null");
        BizSpokenSceneConversationModel bizSpokenSceneConversationModel = this.repository.get(entity.getConversationId());
        if (bizSpokenSceneConversationModel == null || CommonConstant.IsDeleted.Y.equals(bizSpokenSceneConversationModel.getIsDeleted())) {
            return null;
        }
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(entity, bizSpokenSceneConversationModel, copyOptions);
        BizSpokenSceneConversationModel saveModel = this.repository.save(bizSpokenSceneConversationModel);
        return SpokenSceneConversationConvert.modelToEntity(saveModel);
    }

    @Override
    public List<SpokenSceneConversationQueryItem> spokenSceneConversationQuery(SpokenSceneConversationQueryCondition condition, PagingInfo pagingInfo) {
        return this.sqlDao.query(condition, SpokenSceneConversationQueryItem.class, pagingInfo);
    }
}