package cn.com.poc.spokenscene.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.constant.SpokenSceneConstant;
import cn.com.poc.spokenscene.convert.SpokenSceneCategorizationConvert;
import cn.com.poc.spokenscene.convert.SpokenSceneConvert;
import cn.com.poc.spokenscene.entity.BizSpokenSceneCategorizationEntity;
import cn.com.poc.spokenscene.entity.BizSpokenSceneEntity;
import cn.com.poc.spokenscene.model.BizSpokenSceneCategorizationModel;
import cn.com.poc.spokenscene.model.BizSpokenSceneModel;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.poc.spokenscene.service.BizSpokenSceneService;
import cn.com.poc.spokenscene.repository.BizSpokenSceneRepository;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
public class BizSpokenSceneServiceImpl extends BaseServiceImpl
        implements BizSpokenSceneService {

    @Resource
    private BizSpokenSceneRepository repository;

    @Override
    public BizSpokenSceneEntity get(Long id) {
        Assert.notNull(id, "id is null");
        BizSpokenSceneModel bizSpokenSceneModel = this.repository.get(id);
        if (bizSpokenSceneModel == null || bizSpokenSceneModel.getIsDeleted().equals(CommonConstant.IsDeleted.Y)) {
            return null;
        }
        return SpokenSceneConvert.modelToEntity(bizSpokenSceneModel);
    }

    @Override
    public List<BizSpokenSceneEntity> findByExample(BizSpokenSceneEntity entity, PagingInfo pagingInfo) {
        List<BizSpokenSceneEntity> result = new ArrayList<>();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizSpokenSceneModel model = SpokenSceneConvert.entityToModel(entity);
        String orderBy = "sorted asc";
        List<BizSpokenSceneModel> models = this.repository.findByExample(model, orderBy, pagingInfo);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(SpokenSceneConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public void deleteById(Long id) {
        Assert.notNull(id, "id is null");
        BizSpokenSceneModel model = this.repository.get(id);
        if (null == model) {
            return;
        }
        model.setIsDeleted(CommonConstant.IsDeleted.Y);
        repository.save(model);
    }

    @Override
    public BizSpokenSceneEntity save(BizSpokenSceneEntity entity) {
        Assert.notNull(entity, "entity is null");
        Assert.notBlank(entity.getDifficultyLevel(), "difficulty level is null");
        entity.setSpokenSceneId(null);
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizSpokenSceneModel bizSpokenSceneModel = SpokenSceneConvert.entityToModel(entity);
        BizSpokenSceneModel saveModel = this.repository.save(bizSpokenSceneModel);
        return SpokenSceneConvert.modelToEntity(saveModel);
    }

    @Override
    public BizSpokenSceneEntity update(BizSpokenSceneEntity entity) {
        Assert.notNull(entity, "entity is null");
        Assert.notNull(entity.getSpokenSceneId(), "spokenSceneId is null");
        BizSpokenSceneModel bizSpokenSceneModel = this.repository.get(entity.getSpokenSceneId());
        if (bizSpokenSceneModel == null || CommonConstant.IsDeleted.Y.equals(bizSpokenSceneModel.getIsDeleted())) {
            return null;
        }
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(entity, bizSpokenSceneModel, copyOptions);
        BizSpokenSceneModel saveModel = this.repository.save(bizSpokenSceneModel);
        return SpokenSceneConvert.modelToEntity(saveModel);
    }


    @Override
    public boolean checkRenameToTitle(Long sceneCategorizationId, String title, Long filtersSpokenSceneId) {
        Assert.notNull(sceneCategorizationId, "sceneCategorizationId is null");
        Assert.notBlank(title, "title is null");
        BizSpokenSceneModel bizSpokenSceneModel = new BizSpokenSceneModel();
        bizSpokenSceneModel.setSceneCategorizationId(sceneCategorizationId);
        bizSpokenSceneModel.setSpokenSceneTitle(title);
        bizSpokenSceneModel.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizSpokenSceneModel> models = this.repository.findByExample(bizSpokenSceneModel, null);
        if (CollectionUtils.isEmpty(models)) {
            return false;
        }

        if (null != filtersSpokenSceneId) {
            long size = models.stream().filter(model -> !Objects.equals(model.getSpokenSceneId(), filtersSpokenSceneId)).count();
            if (size > 0) {
                return true;
            }
        } else {
            if (CollectionUtils.isNotEmpty(models)) {
                return true;
            }
        }

        return false;
    }
}