package cn.com.poc.spokenscene.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.convert.SpokenSceneWordsConvert;
import cn.com.poc.spokenscene.entity.BizSpokenSceneWordsEntity;
import cn.com.poc.spokenscene.model.BizSpokenSceneWordsModel;
import cn.com.poc.spokenscene.query.SpokenSceneWordsQueryCondition;
import cn.com.poc.spokenscene.query.SpokenSceneWordsQueryItem;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.poc.spokenscene.service.BizSpokenSceneWordsService;
import cn.com.poc.spokenscene.repository.BizSpokenSceneWordsRepository;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
public class BizSpokenSceneWordsServiceImpl extends BaseServiceImpl
        implements BizSpokenSceneWordsService {

    @Resource
    private BizSpokenSceneWordsRepository repository;

    @Override
    public BizSpokenSceneWordsEntity get(Long id) {
        Assert.notNull(id, "id不能为空");
        BizSpokenSceneWordsModel model = this.repository.get(id);
        if (model == null || model.getIsDeleted().equals(CommonConstant.IsDeleted.Y)) {
            return null;
        }
        return SpokenSceneWordsConvert.modelToEntity(model);
    }

    @Override
    public List<BizSpokenSceneWordsEntity> findByExample(BizSpokenSceneWordsEntity entity, PagingInfo pagingInfo) {
        List<BizSpokenSceneWordsEntity> result = new ArrayList<>();
        BizSpokenSceneWordsModel model = SpokenSceneWordsConvert.entityToModel(entity);
        List<BizSpokenSceneWordsModel> models = this.repository.findByExample(model, pagingInfo);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(SpokenSceneWordsConvert::modelToEntity).collect(java.util.stream.Collectors.toList());
        }
        return result;
    }

    @Override
    public void deleteById(Long id) {
        Assert.notNull(id, "id不能为空");
        BizSpokenSceneWordsModel model = this.repository.get(id);
        if (model == null) {
            return;
        }
        model.setIsDeleted(CommonConstant.IsDeleted.Y);
        this.repository.save(model);

    }

    @Override
    public BizSpokenSceneWordsEntity save(BizSpokenSceneWordsEntity entity) {
        Assert.notNull(entity, "entity不能为空");
        entity.setWordId(null);
        BizSpokenSceneWordsModel model = SpokenSceneWordsConvert.entityToModel(entity);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        return SpokenSceneWordsConvert.modelToEntity(this.repository.save(model));
    }

    @Override
    public BizSpokenSceneWordsEntity update(BizSpokenSceneWordsEntity entity) {
        Assert.notNull(entity, "entity不能为空");
        Assert.notNull(entity.getWordId(), "wordId不能为空");
        BizSpokenSceneWordsModel model = this.repository.get(entity.getWordId());
        if (model == null || model.getIsDeleted().equals(CommonConstant.IsDeleted.Y)) {
            return null;
        }
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(entity, model, copyOptions);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        return SpokenSceneWordsConvert.modelToEntity(this.repository.save(model));
    }

    @Override
    public List<SpokenSceneWordsQueryItem> spokenSceneWordsQuery(SpokenSceneWordsQueryCondition condition, PagingInfo pagingInfo) {
        return this.sqlDao.query(condition, SpokenSceneWordsQueryItem.class, pagingInfo);
    }
}