package cn.com.poc.spokenscene.service.impl;

import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.utils.Assert;
import cn.com.poc.spokenscene.convert.SpokenSentencesConvert;
import cn.com.poc.spokenscene.entity.BizSpokenSentencesEntity;
import cn.com.poc.spokenscene.model.BizSpokenSentencesModel;
import cn.com.poc.spokenscene.repository.BizSpokenSentencesRepository;
import cn.com.poc.spokenscene.service.BizSpokenSentencesService;
import cn.com.yict.framemax.core.service.impl.BaseServiceImpl;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class BizSpokenSentencesServiceImpl extends BaseServiceImpl
        implements BizSpokenSentencesService {

    @Resource
    private BizSpokenSentencesRepository repository;

    @Override
    public BizSpokenSentencesEntity get(Long id) {
        Assert.notNull(id, "id is null");
        BizSpokenSentencesModel model = this.repository.get(id);
        if (model == null || model.getIsDeleted().equals(CommonConstant.IsDeleted.Y)) {
            return null;
        }
        return SpokenSentencesConvert.modelToEntity(model);
    }

    @Override
    public List<BizSpokenSentencesEntity> findByExample(BizSpokenSentencesEntity entity, PagingInfo pagingInfo) {
        List<BizSpokenSentencesEntity> result = new ArrayList<>();
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizSpokenSentencesModel model = SpokenSentencesConvert.entityToModel(entity);
        List<BizSpokenSentencesModel> models = this.repository.findByExample(model, pagingInfo);
        if (CollectionUtils.isNotEmpty(models)) {
            result = models.stream().map(SpokenSentencesConvert::modelToEntity).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public void deleteById(Long id) {
        Assert.notNull(id, "id is null");
        BizSpokenSentencesModel model = this.repository.get(id);
        if (null == model) {
            return;
        }
        model.setIsDeleted(CommonConstant.IsDeleted.Y);
        repository.save(model);
    }

    @Override
    public BizSpokenSentencesEntity save(BizSpokenSentencesEntity entity) {
        Assert.notNull(entity, "entity is null");
        entity.setSentencesId(null);
        entity.setIsDeleted(CommonConstant.IsDeleted.N);
        BizSpokenSentencesModel model = SpokenSentencesConvert.entityToModel(entity);
        BizSpokenSentencesModel saveModel = this.repository.save(model);
        return SpokenSentencesConvert.modelToEntity(saveModel);
    }

    @Override
    public BizSpokenSentencesEntity update(BizSpokenSentencesEntity entity) {
        Assert.notNull(entity, "entity is null");
        Assert.notNull(entity.getSentencesId(), "spokenSceneId is null");
        BizSpokenSentencesModel model = this.repository.get(entity.getSentencesId());
        if (model == null || CommonConstant.IsDeleted.Y.equals(model.getIsDeleted())) {
            return null;
        }
        CopyOptions copyOptions = new CopyOptions().ignoreNullValue();
        BeanUtil.copyProperties(entity, model, copyOptions);
        BizSpokenSentencesModel saveModel = this.repository.save(model);
        return SpokenSentencesConvert.modelToEntity(saveModel);
    }
}