package cn.com.poc.thirdparty.resource.tencent.service.impl;

import cn.com.poc.common.entity.media.MediaType;
import cn.com.poc.common.service.BosConfigService;
import cn.com.poc.common.utils.UUIDTool;
import cn.com.poc.thirdparty.resource.tencent.service.TencentService;
import cn.com.yict.framemax.core.config.Config;
import com.tencent.SpeechClient;
import com.tencent.tts.model.SpeechSynthesisRequest;
import com.tencent.tts.model.SpeechSynthesisResponse;
import com.tencent.tts.service.SpeechSynthesisListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;


/**
 * @author alex.yao
 * @date 2023/5/9
 **/
@Service
public class TencentServiceImpl implements TencentService {

    final private Logger logger = LoggerFactory.getLogger(TencentServiceImpl.class);

    /**
     * appId
     */
    final private String appId = Config.get("tencent.speech.synthesizer.appid");
    ;

    /**
     * secretId
     */
    final private String secretId = Config.get("tencent.speech.synthesizer.secretId");

    /**
     * secretkKey
     */
    final private String secretKey = Config.get("tencent.speech.synthesizer.secretKey");

    @Resource
    private BosConfigService bosConfigService;


    @Override
    public String textToAudio(String text, String voiceType) throws Exception {
        CountDownLatch countDownLatch = new CountDownLatch(1);
        StringBuffer audioBase64 = new StringBuffer();

        //创建SpeechSynthesizerClient实例，目前是单例
        com.tencent.SpeechClient client = SpeechClient.newInstance(appId, secretId, secretKey);
        //初始化SpeechSynthesizerRequest，SpeechSynthesizerRequest包含请求参数
        SpeechSynthesisRequest request = SpeechSynthesisRequest.initialize();
        request.setSampleRate(16000);
        request.setCodec(MediaType.voice_mp3.fileSuffix());
        request.setVolume(0);
        request.setVoiceType(Integer.valueOf(voiceType));
        //使用客户端client创建语音合成实例
        client.newSpeechSynthesizer(request, new SpeechSynthesisListener() {

            List<byte[]> audioBytes = new ArrayList<>();

            AtomicInteger sessionId = new AtomicInteger(0);

            @Override
            public void onComplete(SpeechSynthesisResponse response) {
                try {
                    int totalLength = 0;
                    for (byte[] audioByte : audioBytes) {
                        totalLength += audioByte.length;
                    }
                    byte[] bytes = new byte[totalLength];
                    int index = 0;
                    for (byte[] audioByte : audioBytes) {
                        System.arraycopy(audioByte, 0, bytes, index, audioByte.length);
                        index += audioByte.length;
                    }
                    String url = bosConfigService.uploadFileByByteArray2Oss(bytes, UUIDTool.getUUID(), MediaType.voice_mp3.fileSuffix());
                    audioBase64.append(url);
                } catch (Exception e) {
                    throw new RuntimeException("上传文件失败");
                } finally {
                    countDownLatch.countDown();
                }
            }

            @Override
            public void onMessage(byte[] data) {
                sessionId.incrementAndGet();
                audioBytes.add(data);
            }

            @Override
            public void onFail(SpeechSynthesisResponse response) {
                logger.warn("onFail:{}", response.getMessage());
                countDownLatch.countDown();
            }
        }).synthesisLongText(text);
        //等待合成完成
        countDownLatch.await(100, TimeUnit.SECONDS);
        return audioBase64.toString();
    }
}
