package cn.com.poc.thirdparty.resource.tencent.util;

import cn.com.poc.common.utils.DateUtils;
import cn.com.yict.framemax.core.config.Config;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Date;

public class RtmpUrlUtils {

    /**
     * 机器人向 TRTC推流地址
     *
     * @param roomId
     * @param userId
     * @return
     */
    public static String getTRTCPushUrl(String roomId, String userId) {
        long sdkappid = Long.parseLong(Config.get("tencent.rtc.sdkAppId"));
        String key = Config.get("tencent.rtc.secretKey");
        String usersig = new UserSigUtil(sdkappid, key).genUserSig(userId, 86400);
        return new StringBuilder().append("rtmp://rtmp.rtc.qq.com/push/")
                .append(roomId)
                .append("?")
                .append("sdkappid=")
                .append(sdkappid)
                .append("&")
                .append("userid=")
                .append(userId)
                .append("&")
                .append("usersig=")
                .append(usersig)
                .toString();
    }


    /**
     * 获取CDN推流地址
     *
     * @param appName
     * @param streamName
     * @param expried    过期天数
     * @return
     */
    public static String getPushRtmpUrl(String appName, String streamName, int expried) {
        Date date = DateUtils.addDays(new Date(), expried);
        String key = "ec971f778012ff3c48b788bbd85b487c";
        return "rtmp://195988.push.tlivecloud.com/" + appName + "/" + streamName + "?" + getSafeUrl(key, streamName, date.getTime() / 1000);
    }


    /**
     * 获取CDN拉流地址
     *
     * @param appName
     * @param streamName
     * @param expried    过期天数
     * @return
     */
    public static String getPullRtmpUrl(String appName, String streamName, int expried) {
        Date date = DateUtils.addDays(new Date(), expried);
        String key = "TDwfB4iRYWbeADA7G3Aj";
        return "rtmp://wiseman-play.gsstcloud.com/" + appName + "/" + streamName + "?" + getSafeUrl(key, streamName, date.getTime() / 1000);
    }

    public static String getPullFLVUrl(String appName, String streamName, int expried) {
        Date date = DateUtils.addDays(new Date(), expried);
        String key = "TDwfB4iRYWbeADA7G3Aj";
        return "http://wiseman-play.gsstcloud.com/" + appName + "/" + streamName + ".flv?" + getSafeUrl(key, streamName, date.getTime() / 1000);
    }

    public static String getPullHLSUrl(String appName, String streamName, int expried) {
        Date date = DateUtils.addDays(new Date(), expried);
        String key = "TDwfB4iRYWbeADA7G3Aj";
        return "http://wiseman-play.gsstcloud.com/" + appName + "/" + streamName + ".m3u8?" + getSafeUrl(key, streamName, date.getTime() / 1000);
    }

    private static final char[] DIGITS_LOWER =
            {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    /*
     * KEY+ streamName + txTime
     */
    private static String getSafeUrl(String key, String streamName, long txTime) {
        String input = new StringBuilder().
                append(key).
                append(streamName).
                append(Long.toHexString(txTime).toUpperCase()).toString();

        String txSecret = null;
        try {
            MessageDigest messageDigest = MessageDigest.getInstance("MD5");
            txSecret = byteArrayToHexString(
                    messageDigest.digest(input.getBytes("UTF-8")));
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        return txSecret == null ? "" :
                new StringBuilder().
                        append("txSecret=").
                        append(txSecret).
                        append("&").
                        append("txTime=").
                        append(Long.toHexString(txTime).toUpperCase()).
                        toString();
    }

    private static String byteArrayToHexString(byte[] data) {
        char[] out = new char[data.length << 1];

        for (int i = 0, j = 0; i < data.length; i++) {
            out[j++] = DIGITS_LOWER[(0xF0 & data[i]) >>> 4];
            out[j++] = DIGITS_LOWER[0x0F & data[i]];
        }
        return new String(out);
    }

}
